from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.location import Location


T = TypeVar("T", bound="TollCollectionLocation")


@_attrs_define
class TollCollectionLocation:
    """Refers to the physical location where the toll is collected. This can include various structures such as toll
    booths, transponder readers, or number-plate cameras.
    It's important to note that certain toll collection methods, such as vignettes, do not have specific toll collection
    locations associated with them, and therefore this element
    will not be present at all.

    The value of this property is a `Location` that specifies the coordinates of the payment location.

        Attributes:
            location (Location): Location on the Earth Example: {'lat': 52.531677, 'lng': 13.381777}.
            name (str | Unset): A descriptive name of the location.
    """

    location: Location
    name: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        location = self.location.to_dict()

        name = self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "location": location,
        })
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.location import Location

        d = dict(src_dict)
        location = Location.from_dict(d.pop("location"))

        name = d.pop("name", UNSET)

        toll_collection_location = cls(
            location=location,
            name=name,
        )

        toll_collection_location.additional_properties = d
        return toll_collection_location

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.health_response_fail_schema_status import HealthResponseFailSchemaStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="HealthResponseFailSchema")


@_attrs_define
class HealthResponseFailSchema:
    """Returns the health of the service

    Attributes:
        status (HealthResponseFailSchemaStatus | Unset): Health status of the service:

            * `ok` - the service is operating normally
            * `fail` - the service is currently encountering a failure
    """

    status: HealthResponseFailSchemaStatus | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        status: str | Unset = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _status = d.pop("status", UNSET)
        status: HealthResponseFailSchemaStatus | Unset
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = HealthResponseFailSchemaStatus(_status)

        health_response_fail_schema = cls(
            status=status,
        )

        health_response_fail_schema.additional_properties = d
        return health_response_fail_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.pedestrian_section_type import PedestrianSectionType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.arrive_action import ArriveAction
    from ..models.base_action import BaseAction
    from ..models.base_summary import BaseSummary
    from ..models.continue_action import ContinueAction
    from ..models.deboard_action import DeboardAction
    from ..models.depart_action import DepartAction
    from ..models.exit_action import ExitAction
    from ..models.offset_action import OffsetAction
    from ..models.passthrough import Passthrough
    from ..models.pedestrian_departure import PedestrianDeparture
    from ..models.pedestrian_notice import PedestrianNotice
    from ..models.pedestrian_span import PedestrianSpan
    from ..models.pedestrian_summary import PedestrianSummary
    from ..models.pedestrian_transport import PedestrianTransport
    from ..models.ref_replacements import RefReplacements
    from ..models.simple_turn_action import SimpleTurnAction
    from ..models.wait_action import WaitAction


T = TypeVar("T", bound="PedestrianSection")


@_attrs_define
class PedestrianSection:
    """Represent a section of a route

    Attributes:
        id (str): Unique identifier of the section
        type_ (PedestrianSectionType): Section type used by the client to identify what extension to the BaseSection are
            available.
        departure (PedestrianDeparture): Departure of pedestrian
        arrival (PedestrianDeparture): Departure of pedestrian
        transport (PedestrianTransport): Information about a transport
        pre_actions (list[BaseAction] | Unset): Actions that must be done prior to `departure`.
        actions (list[ArriveAction | ContinueAction | DepartAction | ExitAction | SimpleTurnAction] | Unset): Actions to
            be performed at or during a specific portion of a section.

            Action offsets are the coordinate index in the polyline.

            *NOTE:* currentRoad and nextRoad are not populated for actions.
        language (str | Unset): Language of the localized strings in the section, if any, in BCP47 format.
        post_actions (list[DeboardAction | WaitAction] | Unset): Actions that must be done after `arrival`.
        turn_by_turn_actions (list[OffsetAction] | Unset): Actions for turn by turn guidance during the travel portion
            of the section, i.e., between `departure` and `arrival`.
        passthrough (list[Passthrough] | Unset): List of via waypoints this section is passing through.

            Each via waypoint of the request that is a `passThrough=true` waypoint, appears as a
            `Passthrough` in the response. It appears in the section that starts with the closest
            non-passthrough via specified before it or origin.

            The passthrough vias appear in this list in the order they are traversed. They are
            traversed in the order they are specified in the request.
        summary (PedestrianSummary | Unset): Total value of key attributes for a route section.
        travel_summary (BaseSummary | Unset): Total value of key attributes for a route section.
        polyline (str | Unset): Line string in [Flexible Polyline](https://github.com/heremaps/flexible-polyline)
            format. Coordinates are in the WGS84 coordinate system, including `Elevation` (if present). Example:
            A05xgKuy2xCx9B7vUl0OhnR54EqSzpEl-HxjD3pBiGnyGi2CvwFsgD3nD4vB6e.
        notices (list[PedestrianNotice] | Unset): Contains a list of issues related to this section of the route.
        spans (list[PedestrianSpan] | Unset): Spans attached to a `Section` describing pedestrian content.
        ref_replacements (RefReplacements | Unset): Dictionary of placeholders to replacement strings for the compact
            representation of map entity references.
    """

    id: str
    type_: PedestrianSectionType
    departure: PedestrianDeparture
    arrival: PedestrianDeparture
    transport: PedestrianTransport
    pre_actions: list[BaseAction] | Unset = UNSET
    actions: (
        list[
            ArriveAction | ContinueAction | DepartAction | ExitAction | SimpleTurnAction
        ]
        | Unset
    ) = UNSET
    language: str | Unset = UNSET
    post_actions: list[DeboardAction | WaitAction] | Unset = UNSET
    turn_by_turn_actions: list[OffsetAction] | Unset = UNSET
    passthrough: list[Passthrough] | Unset = UNSET
    summary: PedestrianSummary | Unset = UNSET
    travel_summary: BaseSummary | Unset = UNSET
    polyline: str | Unset = UNSET
    notices: list[PedestrianNotice] | Unset = UNSET
    spans: list[PedestrianSpan] | Unset = UNSET
    ref_replacements: RefReplacements | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.arrive_action import ArriveAction
        from ..models.continue_action import ContinueAction
        from ..models.depart_action import DepartAction
        from ..models.exit_action import ExitAction
        from ..models.wait_action import WaitAction

        id = self.id

        type_ = self.type_.value

        departure = self.departure.to_dict()

        arrival = self.arrival.to_dict()

        transport = self.transport.to_dict()

        pre_actions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.pre_actions, Unset):
            pre_actions = []
            for pre_actions_item_data in self.pre_actions:
                pre_actions_item = pre_actions_item_data.to_dict()
                pre_actions.append(pre_actions_item)

        actions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.actions, Unset):
            actions = []
            for actions_item_data in self.actions:
                actions_item: dict[str, Any]
                if isinstance(actions_item_data, DepartAction):
                    actions_item = actions_item_data.to_dict()
                elif isinstance(actions_item_data, ArriveAction):
                    actions_item = actions_item_data.to_dict()
                elif isinstance(actions_item_data, ContinueAction):
                    actions_item = actions_item_data.to_dict()
                elif isinstance(actions_item_data, ExitAction):
                    actions_item = actions_item_data.to_dict()
                else:
                    actions_item = actions_item_data.to_dict()

                actions.append(actions_item)

        language = self.language

        post_actions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.post_actions, Unset):
            post_actions = []
            for post_actions_item_data in self.post_actions:
                post_actions_item: dict[str, Any]
                if isinstance(post_actions_item_data, WaitAction):
                    post_actions_item = post_actions_item_data.to_dict()
                else:
                    post_actions_item = post_actions_item_data.to_dict()

                post_actions.append(post_actions_item)

        turn_by_turn_actions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.turn_by_turn_actions, Unset):
            turn_by_turn_actions = []
            for turn_by_turn_actions_item_data in self.turn_by_turn_actions:
                turn_by_turn_actions_item = turn_by_turn_actions_item_data.to_dict()
                turn_by_turn_actions.append(turn_by_turn_actions_item)

        passthrough: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.passthrough, Unset):
            passthrough = []
            for passthrough_item_data in self.passthrough:
                passthrough_item = passthrough_item_data.to_dict()
                passthrough.append(passthrough_item)

        summary: dict[str, Any] | Unset = UNSET
        if not isinstance(self.summary, Unset):
            summary = self.summary.to_dict()

        travel_summary: dict[str, Any] | Unset = UNSET
        if not isinstance(self.travel_summary, Unset):
            travel_summary = self.travel_summary.to_dict()

        polyline = self.polyline

        notices: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.notices, Unset):
            notices = []
            for notices_item_data in self.notices:
                notices_item = notices_item_data.to_dict()
                notices.append(notices_item)

        spans: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.spans, Unset):
            spans = []
            for spans_item_data in self.spans:
                spans_item = spans_item_data.to_dict()
                spans.append(spans_item)

        ref_replacements: dict[str, Any] | Unset = UNSET
        if not isinstance(self.ref_replacements, Unset):
            ref_replacements = self.ref_replacements.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "type": type_,
            "departure": departure,
            "arrival": arrival,
            "transport": transport,
        })
        if pre_actions is not UNSET:
            field_dict["preActions"] = pre_actions
        if actions is not UNSET:
            field_dict["actions"] = actions
        if language is not UNSET:
            field_dict["language"] = language
        if post_actions is not UNSET:
            field_dict["postActions"] = post_actions
        if turn_by_turn_actions is not UNSET:
            field_dict["turnByTurnActions"] = turn_by_turn_actions
        if passthrough is not UNSET:
            field_dict["passthrough"] = passthrough
        if summary is not UNSET:
            field_dict["summary"] = summary
        if travel_summary is not UNSET:
            field_dict["travelSummary"] = travel_summary
        if polyline is not UNSET:
            field_dict["polyline"] = polyline
        if notices is not UNSET:
            field_dict["notices"] = notices
        if spans is not UNSET:
            field_dict["spans"] = spans
        if ref_replacements is not UNSET:
            field_dict["refReplacements"] = ref_replacements

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.arrive_action import ArriveAction
        from ..models.base_action import BaseAction
        from ..models.base_summary import BaseSummary
        from ..models.continue_action import ContinueAction
        from ..models.deboard_action import DeboardAction
        from ..models.depart_action import DepartAction
        from ..models.exit_action import ExitAction
        from ..models.offset_action import OffsetAction
        from ..models.passthrough import Passthrough
        from ..models.pedestrian_departure import PedestrianDeparture
        from ..models.pedestrian_notice import PedestrianNotice
        from ..models.pedestrian_span import PedestrianSpan
        from ..models.pedestrian_summary import PedestrianSummary
        from ..models.pedestrian_transport import PedestrianTransport
        from ..models.ref_replacements import RefReplacements
        from ..models.simple_turn_action import SimpleTurnAction
        from ..models.wait_action import WaitAction

        d = dict(src_dict)
        id = d.pop("id")

        type_ = PedestrianSectionType(d.pop("type"))

        departure = PedestrianDeparture.from_dict(d.pop("departure"))

        arrival = PedestrianDeparture.from_dict(d.pop("arrival"))

        transport = PedestrianTransport.from_dict(d.pop("transport"))

        _pre_actions = d.pop("preActions", UNSET)
        pre_actions: list[BaseAction] | Unset = UNSET
        if _pre_actions is not UNSET:
            pre_actions = []
            for pre_actions_item_data in _pre_actions:
                pre_actions_item = BaseAction.from_dict(pre_actions_item_data)

                pre_actions.append(pre_actions_item)

        _actions = d.pop("actions", UNSET)
        actions: (
            list[
                ArriveAction
                | ContinueAction
                | DepartAction
                | ExitAction
                | SimpleTurnAction
            ]
            | Unset
        ) = UNSET
        if _actions is not UNSET:
            actions = []
            for actions_item_data in _actions:

                def _parse_actions_item(
                    data: object,
                ) -> (
                    ArriveAction
                    | ContinueAction
                    | DepartAction
                    | ExitAction
                    | SimpleTurnAction
                ):
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_pedestrian_action_type_0 = (
                            DepartAction.from_dict(data)
                        )

                        return componentsschemas_pedestrian_action_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_pedestrian_action_type_1 = (
                            ArriveAction.from_dict(data)
                        )

                        return componentsschemas_pedestrian_action_type_1
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_pedestrian_action_type_2 = (
                            ContinueAction.from_dict(data)
                        )

                        return componentsschemas_pedestrian_action_type_2
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_pedestrian_action_type_3 = (
                            ExitAction.from_dict(data)
                        )

                        return componentsschemas_pedestrian_action_type_3
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_pedestrian_action_type_4 = (
                        SimpleTurnAction.from_dict(data)
                    )

                    return componentsschemas_pedestrian_action_type_4

                actions_item = _parse_actions_item(actions_item_data)

                actions.append(actions_item)

        language = d.pop("language", UNSET)

        _post_actions = d.pop("postActions", UNSET)
        post_actions: list[DeboardAction | WaitAction] | Unset = UNSET
        if _post_actions is not UNSET:
            post_actions = []
            for post_actions_item_data in _post_actions:

                def _parse_post_actions_item(
                    data: object,
                ) -> DeboardAction | WaitAction:
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_pedestrian_post_action_type_0 = (
                            WaitAction.from_dict(data)
                        )

                        return componentsschemas_pedestrian_post_action_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_pedestrian_post_action_type_1 = (
                        DeboardAction.from_dict(data)
                    )

                    return componentsschemas_pedestrian_post_action_type_1

                post_actions_item = _parse_post_actions_item(post_actions_item_data)

                post_actions.append(post_actions_item)

        _turn_by_turn_actions = d.pop("turnByTurnActions", UNSET)
        turn_by_turn_actions: list[OffsetAction] | Unset = UNSET
        if _turn_by_turn_actions is not UNSET:
            turn_by_turn_actions = []
            for turn_by_turn_actions_item_data in _turn_by_turn_actions:
                turn_by_turn_actions_item = OffsetAction.from_dict(
                    turn_by_turn_actions_item_data
                )

                turn_by_turn_actions.append(turn_by_turn_actions_item)

        _passthrough = d.pop("passthrough", UNSET)
        passthrough: list[Passthrough] | Unset = UNSET
        if _passthrough is not UNSET:
            passthrough = []
            for passthrough_item_data in _passthrough:
                passthrough_item = Passthrough.from_dict(passthrough_item_data)

                passthrough.append(passthrough_item)

        _summary = d.pop("summary", UNSET)
        summary: PedestrianSummary | Unset
        if isinstance(_summary, Unset):
            summary = UNSET
        else:
            summary = PedestrianSummary.from_dict(_summary)

        _travel_summary = d.pop("travelSummary", UNSET)
        travel_summary: BaseSummary | Unset
        if isinstance(_travel_summary, Unset):
            travel_summary = UNSET
        else:
            travel_summary = BaseSummary.from_dict(_travel_summary)

        polyline = d.pop("polyline", UNSET)

        _notices = d.pop("notices", UNSET)
        notices: list[PedestrianNotice] | Unset = UNSET
        if _notices is not UNSET:
            notices = []
            for notices_item_data in _notices:
                notices_item = PedestrianNotice.from_dict(notices_item_data)

                notices.append(notices_item)

        _spans = d.pop("spans", UNSET)
        spans: list[PedestrianSpan] | Unset = UNSET
        if _spans is not UNSET:
            spans = []
            for spans_item_data in _spans:
                spans_item = PedestrianSpan.from_dict(spans_item_data)

                spans.append(spans_item)

        _ref_replacements = d.pop("refReplacements", UNSET)
        ref_replacements: RefReplacements | Unset
        if isinstance(_ref_replacements, Unset):
            ref_replacements = UNSET
        else:
            ref_replacements = RefReplacements.from_dict(_ref_replacements)

        pedestrian_section = cls(
            id=id,
            type_=type_,
            departure=departure,
            arrival=arrival,
            transport=transport,
            pre_actions=pre_actions,
            actions=actions,
            language=language,
            post_actions=post_actions,
            turn_by_turn_actions=turn_by_turn_actions,
            passthrough=passthrough,
            summary=summary,
            travel_summary=travel_summary,
            polyline=polyline,
            notices=notices,
            spans=spans,
            ref_replacements=ref_replacements,
        )

        pedestrian_section.additional_properties = d
        return pedestrian_section

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.violated_transport_mode_type import ViolatedTransportModeType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ViolatedTransportMode")


@_attrs_define
class ViolatedTransportMode:
    """Contains details about the violated transport mode on this road, such as trucks being generally forbidden.

    Attributes:
        type_ (ViolatedTransportModeType): Detail type. Each type of detail might contain extra attributes.

            **NOTE:** The list of possible detail types may be extended in the future.
            The client application is expected to handle such a case gracefully.
        title (str | Unset): Detail title
        cause (str | Unset): Cause of the notice
    """

    type_: ViolatedTransportModeType
    title: str | Unset = UNSET
    cause: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        title = self.title

        cause = self.cause

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
        })
        if title is not UNSET:
            field_dict["title"] = title
        if cause is not UNSET:
            field_dict["cause"] = cause

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = ViolatedTransportModeType(d.pop("type"))

        title = d.pop("title", UNSET)

        cause = d.pop("cause", UNSET)

        violated_transport_mode = cls(
            type_=type_,
            title=title,
            cause=cause,
        )

        violated_transport_mode.additional_properties = d
        return violated_transport_mode

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.notice_severity import NoticeSeverity
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.base_notice_detail import BaseNoticeDetail


T = TypeVar("T", bound="IsolineResponseNotice")


@_attrs_define
class IsolineResponseNotice:
    """A notice contains important notifications.

    Attributes:
        code (str): Extensible enum: `avoidSegmentsInvalidId` `avoidSegmentsInvalidPrefix` `avoidZonesInvalidId`
            `avoidTruckRoadTypesInvalidId` `cancelled` `couldNotMatchOrigin` `couldNotMatchDestination`
            `isolineCalculationFailed` `isolineCalculationBlocked` `unknownParameter` `...`
            Currently known codes (non-exhaustive: this list will be extended for new error situations):

            | Code      | Description  | Severity |
            | --------- | ------- | ------- |
            | avoidSegmentsInvalidId | Avoid segments: The provided segment ID was not found | info |
            | avoidSegmentsInvalidPrefix | Avoid segments: The provided domain prefix was not found | info |
            | avoidZonesInvalidId | Avoid zones: The provided zone ID was not found | info |
            | avoidTruckRoadTypesInvalidId | The provided truck road type id was not found | info |
            | cancelled | Calculation took too long and was cancelled | critical |
            | couldNotMatchOrigin | Origin waypoint could not be matched | critical |
            | couldNotMatchDestination | Destination waypoint could not be matched | critical |
            | isolineCalculationFailed | Isolines could not be calculated. | critical |
            | isolineCalculationBlocked | Isolines calculation was blocked by restrictions or traffic at the requested
            origin or destination | critical |
            | unknownParameter | The provided parameter is unknown | info |
             Example: noRouteFound.
        title (str | Unset): Human-readable notice description. Example: No route found.
        severity (NoticeSeverity | Unset): Describes the impact a notice has on the resource to which the notice is
            attached.
            * critical - The notice must not be ignored, even if the type of notice is not known to the user. Any associated
            resource (e.g., route section) must not be used without further evaluation.
            * info - The notice is for informative purposes, but does not affect usability of the route.
        details (list[BaseNoticeDetail] | Unset): Additional details about the notice
    """

    code: str
    title: str | Unset = UNSET
    severity: NoticeSeverity | Unset = UNSET
    details: list[BaseNoticeDetail] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        code = self.code

        title = self.title

        severity: str | Unset = UNSET
        if not isinstance(self.severity, Unset):
            severity = self.severity.value

        details: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.details, Unset):
            details = []
            for details_item_data in self.details:
                details_item = details_item_data.to_dict()
                details.append(details_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "code": code,
        })
        if title is not UNSET:
            field_dict["title"] = title
        if severity is not UNSET:
            field_dict["severity"] = severity
        if details is not UNSET:
            field_dict["details"] = details

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.base_notice_detail import BaseNoticeDetail

        d = dict(src_dict)
        code = d.pop("code")

        title = d.pop("title", UNSET)

        _severity = d.pop("severity", UNSET)
        severity: NoticeSeverity | Unset
        if isinstance(_severity, Unset):
            severity = UNSET
        else:
            severity = NoticeSeverity(_severity)

        _details = d.pop("details", UNSET)
        details: list[BaseNoticeDetail] | Unset = UNSET
        if _details is not UNSET:
            details = []
            for details_item_data in _details:
                details_item = BaseNoticeDetail.from_dict(details_item_data)

                details.append(details_item)

        isoline_response_notice = cls(
            code=code,
            title=title,
            severity=severity,
            details=details,
        )

        isoline_response_notice.additional_properties = d
        return isoline_response_notice

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.toll_collection_location import TollCollectionLocation
    from ..models.toll_fare import TollFare


T = TypeVar("T", bound="TollCost")


@_attrs_define
class TollCost:
    """Information for a single toll payment.

    Attributes:
        toll_system (str): The name of the toll system collecting the toll.

            **NOTE** This property is deprecated. For a toll cost associated with multiple toll systems this parameter will
            return only one toll system name. Use `tollSystems` property to get all the systems.
        toll_system_ref (int): Reference index of the affected toll system in the `tollSystems` array.

            **NOTE** This property is deprecated. For a toll cost associated with multiple systems this parameter will
            return only one system. Use `tollSystems` property to get all the systems.
        fares (list[TollFare]): The list of possible `Fare`s represents the various fares that may apply for the tolls.
            The specific fares can vary based on factors such as the time of day, payment method, and vehicle
            characteristics.

            **Note**: The router presents the relevant fare options based on the original query, on a best effort basis.
            The `Fare` object for tolls will always be of type `SinglePrice`, indicating a single price for the toll.
        toll_systems (list[int] | Unset): Reference indices of the associated toll system(s). These indices correspond
            to the `tollSystems` array in the enclosing section.

            A toll cost may be associated with multiple systems. For details, refer to this Developer Guide
            [tutorial](https://www.here.com/docs/bundle/routing-api-developer-guide-v8/page/tutorials/tolls-multiple-
            systems.html).
        country_code (str | Unset): ISO-3166-1 alpha-3 code Example: FRA.
        toll_collection_locations (list[TollCollectionLocation] | Unset): The toll places represent the location(s)
            where the fare is collected. For tolls measured by distance, both the entry and exit toll locations are
            returned. It's important to note that while both entry and exit toll locations are provided, the payment is
            typically made at only one of these places, which is usually the exit toll location.
    """

    toll_system: str
    toll_system_ref: int
    fares: list[TollFare]
    toll_systems: list[int] | Unset = UNSET
    country_code: str | Unset = UNSET
    toll_collection_locations: list[TollCollectionLocation] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        toll_system = self.toll_system

        toll_system_ref = self.toll_system_ref

        fares = []
        for fares_item_data in self.fares:
            fares_item = fares_item_data.to_dict()
            fares.append(fares_item)

        toll_systems: list[int] | Unset = UNSET
        if not isinstance(self.toll_systems, Unset):
            toll_systems = self.toll_systems

        country_code = self.country_code

        toll_collection_locations: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.toll_collection_locations, Unset):
            toll_collection_locations = []
            for toll_collection_locations_item_data in self.toll_collection_locations:
                toll_collection_locations_item = (
                    toll_collection_locations_item_data.to_dict()
                )
                toll_collection_locations.append(toll_collection_locations_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "tollSystem": toll_system,
            "tollSystemRef": toll_system_ref,
            "fares": fares,
        })
        if toll_systems is not UNSET:
            field_dict["tollSystems"] = toll_systems
        if country_code is not UNSET:
            field_dict["countryCode"] = country_code
        if toll_collection_locations is not UNSET:
            field_dict["tollCollectionLocations"] = toll_collection_locations

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.toll_collection_location import TollCollectionLocation
        from ..models.toll_fare import TollFare

        d = dict(src_dict)
        toll_system = d.pop("tollSystem")

        toll_system_ref = d.pop("tollSystemRef")

        fares = []
        _fares = d.pop("fares")
        for fares_item_data in _fares:
            fares_item = TollFare.from_dict(fares_item_data)

            fares.append(fares_item)

        toll_systems = cast(list[int], d.pop("tollSystems", UNSET))

        country_code = d.pop("countryCode", UNSET)

        _toll_collection_locations = d.pop("tollCollectionLocations", UNSET)
        toll_collection_locations: list[TollCollectionLocation] | Unset = UNSET
        if _toll_collection_locations is not UNSET:
            toll_collection_locations = []
            for toll_collection_locations_item_data in _toll_collection_locations:
                toll_collection_locations_item = TollCollectionLocation.from_dict(
                    toll_collection_locations_item_data
                )

                toll_collection_locations.append(toll_collection_locations_item)

        toll_cost = cls(
            toll_system=toll_system,
            toll_system_ref=toll_system_ref,
            fares=fares,
            toll_systems=toll_systems,
            country_code=country_code,
            toll_collection_locations=toll_collection_locations,
        )

        toll_cost.additional_properties = d
        return toll_cost

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PostDiscoverShowItem(str, Enum):
    EMOBILITYSERVICEPROVIDERS = "eMobilityServiceProviders"
    EV = "ev"
    FUEL = "fuel"
    FUELPRICES = "fuelPrices"
    PHONEMES = "phonemes"
    STREETINFO = "streetInfo"
    TRIPADVISOR = "tripadvisor"
    TRIPADVISORIMAGEVARIANTS = "tripadvisorImageVariants"
    TRUCK = "truck"
    TZ = "tz"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.notice import Notice
    from ..models.pedestrian_section import PedestrianSection
    from ..models.route_label import RouteLabel
    from ..models.transit_section import TransitSection
    from ..models.vehicle_section import VehicleSection


T = TypeVar("T", bound="RouterRoute")


@_attrs_define
class RouterRoute:
    """A basic route. Includes personal vehicles as car, truck, etc... For all modes, cf. `transportMode`.

    Attributes:
        id (str): Unique identifier of the route
        sections (list[PedestrianSection | TransitSection | VehicleSection]): An ordered list of vehicle, transit, and
            pedestrian sections making up the route.
        notices (list[Notice] | Unset): Contains a list of issues encountered during the processing of this response.
        route_labels (list[RouteLabel] | Unset): Contains a list of the most important names and route numbers on this
            route that differentiate it from other alternatives.
            These names are used to make labels for the main and alternative routes, like "route1 via A4,D10", "route2 via
            D11,5"
            The generated list is expected to be unique for each route in response (but it's not guaranteed)
        route_handle (str | Unset): Opaque handle of the calculated route.

            A handle encodes the calculated route. The route can be decoded from a handle at a
            later point in time as long as the service uses the same map data which was used
            during encoding.

            To request a handle set the `routeHandle` flag in `return` parameter. If a handle is
            requested, but fails to be calculated for any reason, then the `routeHandle` property is
            not available in the response. The rest of the route is intact.
    """

    id: str
    sections: list[PedestrianSection | TransitSection | VehicleSection]
    notices: list[Notice] | Unset = UNSET
    route_labels: list[RouteLabel] | Unset = UNSET
    route_handle: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.pedestrian_section import PedestrianSection
        from ..models.vehicle_section import VehicleSection

        id = self.id

        sections = []
        for sections_item_data in self.sections:
            sections_item: dict[str, Any]
            if isinstance(sections_item_data, VehicleSection):
                sections_item = sections_item_data.to_dict()
            elif isinstance(sections_item_data, PedestrianSection):
                sections_item = sections_item_data.to_dict()
            else:
                sections_item = sections_item_data.to_dict()

            sections.append(sections_item)

        notices: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.notices, Unset):
            notices = []
            for notices_item_data in self.notices:
                notices_item = notices_item_data.to_dict()
                notices.append(notices_item)

        route_labels: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.route_labels, Unset):
            route_labels = []
            for route_labels_item_data in self.route_labels:
                route_labels_item = route_labels_item_data.to_dict()
                route_labels.append(route_labels_item)

        route_handle = self.route_handle

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "sections": sections,
        })
        if notices is not UNSET:
            field_dict["notices"] = notices
        if route_labels is not UNSET:
            field_dict["routeLabels"] = route_labels
        if route_handle is not UNSET:
            field_dict["routeHandle"] = route_handle

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.notice import Notice
        from ..models.pedestrian_section import PedestrianSection
        from ..models.route_label import RouteLabel
        from ..models.transit_section import TransitSection
        from ..models.vehicle_section import VehicleSection

        d = dict(src_dict)
        id = d.pop("id")

        sections = []
        _sections = d.pop("sections")
        for sections_item_data in _sections:

            def _parse_sections_item(
                data: object,
            ) -> PedestrianSection | TransitSection | VehicleSection:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_router_section_type_0 = VehicleSection.from_dict(
                        data
                    )

                    return componentsschemas_router_section_type_0
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_router_section_type_1 = (
                        PedestrianSection.from_dict(data)
                    )

                    return componentsschemas_router_section_type_1
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_router_section_type_2 = TransitSection.from_dict(data)

                return componentsschemas_router_section_type_2

            sections_item = _parse_sections_item(sections_item_data)

            sections.append(sections_item)

        _notices = d.pop("notices", UNSET)
        notices: list[Notice] | Unset = UNSET
        if _notices is not UNSET:
            notices = []
            for notices_item_data in _notices:
                notices_item = Notice.from_dict(notices_item_data)

                notices.append(notices_item)

        _route_labels = d.pop("routeLabels", UNSET)
        route_labels: list[RouteLabel] | Unset = UNSET
        if _route_labels is not UNSET:
            route_labels = []
            for route_labels_item_data in _route_labels:
                route_labels_item = RouteLabel.from_dict(route_labels_item_data)

                route_labels.append(route_labels_item)

        route_handle = d.pop("routeHandle", UNSET)

        router_route = cls(
            id=id,
            sections=sections,
            notices=notices,
            route_labels=route_labels,
            route_handle=route_handle,
        )

        router_route.additional_properties = d
        return router_route

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TruckEngineType(str, Enum):
    ELECTRIC = "electric"
    INTERNALCOMBUSTION = "internalCombustion"
    PLUGINHYBRID = "pluginHybrid"

    def __str__(self) -> str:
        return str(self.value)



import math
from collections.abc import Sequence
from dataclasses import KW_ONLY, dataclass
from typing import Any

import shapely
from geopandas import GeoDataFrame
from pandas import DataFrame
from shapely import MultiPoint, Point, Polygon

from . import _consts, _util
from ._client import CasaGeoClient
from ._errors import CasaGeoError
from ._types import CasaGeoResult


def _qq(fields: dict[str, str | None]) -> str:
    return ";".join(f"{key}={val}" for key, val in fields.items() if val is not None)


def _coords(pos: dict) -> tuple[float, float]:
    return (pos.get("lng", math.nan), pos.get("lat", math.nan))


def _item_position(item: dict) -> Point | None:
    try:
        return Point(_coords(item["position"]))
    except KeyError:
        return None


def _item_navigation(item: dict) -> MultiPoint | None:
    try:
        return MultiPoint([_coords(pos) for pos in item["access"]])
    except KeyError:
        return None


def _item_mapview(item: dict) -> Polygon | None:
    try:
        mv = item["mapView"]
        return shapely.box(mv["west"], mv["south"], mv["east"], mv["north"])
    except KeyError:
        return None


class AddressResult(CasaGeoResult):
    """
    Represents the result of a CasaGeoCoder.address() query.
    """

    def dataframe(
        self,
        id_: Any | None = None,
        *,
        address_details: bool = False,
        coordinates: bool = False,
        match_quality: bool = False,
    ) -> GeoDataFrame:
        """
        Return the result as a GeoDataFrame.

        The dataframe will contain the following columns:

        - *id* (str | Any): Fixed identifier added to each row. Can be
          overridden using the ``id_`` parameter.
        - *subid* (int): Numeric index of the result value starting from zero.
        - *address* (str): Localized display name of this result item.
        - *hereid* (str): Unique identifier for the result item in the HERE
          database.
        - *politicalview* (str): ISO 3166-1 alpha-3 country code representing
          the political view of the query.
        - *resulttype* (str): The type of the result item, one of "addressBlock",
          "administrativeArea", "houseNumber", "intersection", "locality",
          "place", "postalCodePoint" or "street".
        - *position* (Point): The coordinates of a pin on a map corresponding to
          the item.
        - *navigation* (MultiPoint): The coordinates of the item’s registered
          navigation points.
        - *distance* (int): The distance from the search context position to the
          result item in meters.
        - *relevance* (float): The relevance of the result to the query as a
          number between 0 and 1.
        - *timestamp* (datetime): Timestamp of when this result was created.

        When ``address_details`` is ``True``, the dataframe also contains the
        following columns:

        - *postaladdress* (str): Full address of the item, formatted
          according to the regional postal rules. May not include all address
          components.
        - *country* (str): Name of the country.
        - *countrycode* (str): ISO 3166-1 alpha-3 country code of the country.
        - *state* (str): Name of the state within the country.
        - *statecode* (str): Code or abbreviation of the state.
        - *county* (str): Name of the county or subdivision within the state.
        - *countycode* (str): Code or abbreviation of the county.
        - *city* (str): Name of the city.
        - *district* (str): Name of the district within the city.
        - *subdistrict* (str): Name of the subdistrict within the city.
        - *street* (str): Name of the street.
        - *streets* (list[str]): List of street names on an intersection.
        - *block* (str): Name of the city block.
        - *subblock* (str): Name of the subblock within the city block.
        - *postalcode* (str): Postal code associated with the item.
        - *housenumber* (str): House number, including associated qualifiers.
        - *building* (str): Name of the building.
        - *unit* (str): Information about the unit within the building.

        When ``coordinates`` is True, the dataframe also contains the following
        columns:

        - *position_lng* (float): Longitude of the item’s position.
        - *position_lat* (float): Latitude of the item’s position.
        - *navigation_lng* (list[float]): Longitudes of the item’s navigation
          points.
        - *navigation_lat* (list[float]): Latitudes of the item’s navigation
          points.

        When ``match_quality`` is ``True``, the dataframe also contains the
        columns below. Each of the values is a number between 0 and 1,
        indicating how well the output field matches the corresponding input
        field.

        - *mq_country* (float): Match quality of the ``country`` field.
        - *mq_countrycode* (float): Match quality of the ``countrycode`` field.
        - *mq_state* (float): Match quality of the ``state`` field.
        - *mq_statecode* (float): Match quality of the ``statecode`` field.
        - *mq_county* (float): Match quality of the ``county`` field.
        - *mq_countycode* (float): Match quality of the ``countycode`` field.
        - *mq_city* (float): Match quality of the ``city`` field.
        - *mq_district* (float): Match quality of the ``district`` field.
        - *mq_subdistrict* (float): Match quality of the ``subdistrict`` field.
        - *mq_streets* (list[float]): Match quality of the ``streets`` field
          against each street name in the input query.
        - *mq_block* (float): Match quality of the ``block`` field.
        - *mq_subblock* (float): Match quality of the ``subblock`` field.
        - *mq_postalcode* (float): Match quality of the ``postalcode`` field.
        - *mq_housenumber* (float): Match quality of the ``housenumber`` field.
          If the requested house number could not be found, this indicates the
          numeric difference between the requested house number and the returned
          house number.
        - *mq_building* (float): Match quality of the ``building`` field.
        - *mq_unit* (float): Match quality of the ``unit`` field.
        - *mq_placename* (float): Match quality of the result place name against
          the input query.
        - *mq_ontologyname* (float): Match quality of the result ontology name
          against the input query.

        Args:
            id_: Fixed identifier to be added to each row.
            address_details: Include additional address details in the result.
            coordinates: Include numeric coordinate columns in the result.
            match_quality: Include match quality scores in the result.

        Returns:
            GeoDataFrame: The list of determined locations as a GeoDataFrame.
        """
        #             mq_street: float
        #                 The match quality of the determined streets as a number from 0
        #                 to 1. If the input contains multiple street names, the field
        #                 score is calculated for each of them individually, but only the
        #                 first result is returned.

        if id_ is None:
            id_ = str(self._uuid)

        data: list[dict] = []
        for index, item in enumerate(self._data.get("items", [])):
            data.append(row := {})

            address = item.get("address", {})
            scoring = item.get("scoring", {})

            if True:
                # fmt: off
                row["id"]               = id_
                row["subid"]            = index
                row["address"]          = item.get("title")
                row["hereid"]           = item.get("id")
                row["politicalview"]    = item.get("politicalView")
                row["resulttype"]       = item.get("resultType")
                row["position"]         = _item_position(item)
                row["navigation"]       = _item_navigation(item)
                row["distance"]         = item.get("distance")
                row["relevance"]        = scoring.get("queryScore")
                row["timestamp"]        = self._timestamp
                # fmt: on

            if address_details:
                # fmt: off
                row["postaladdress"]    = address.get("label")
                row["country"]          = address.get("countryName")
                row["countrycode"]      = address.get("countryCode")
                row["state"]            = address.get("state")
                row["statecode"]        = address.get("stateCode")
                row["county"]           = address.get("county")
                row["countycode"]       = address.get("countyCode")
                row["city"]             = address.get("city")
                row["district"]         = address.get("district")
                row["subdistrict"]      = address.get("subdistrict")
                row["street"]           = address.get("street")
                row["streets"]          = address.get("streets")
                row["block"]            = address.get("block")
                row["subblock"]         = address.get("subblock")
                row["postalcode"]       = address.get("postalCode")
                row["housenumber"]      = address.get("houseNumber")
                row["building"]         = address.get("building")
                row["unit"]             = address.get("unit")
                # fmt: on

            if coordinates:
                position = item.get("position", {})
                access = item.get("access", None)
                # fmt: off
                row["position_lng"]     = position.get("lng")
                row["position_lat"]     = position.get("lat")
                row["navigation_lng"]   = [p.get("lng", math.nan) for p in access] if access is not None else None
                row["navigation_lat"]   = [p.get("lat", math.nan) for p in access] if access is not None else None
                # fmt: on

            if match_quality:
                score = scoring.get("fieldScore", {})
                # fmt: off
                row["mq_country"]       = score.get("countryName")
                row["mq_countrycode"]   = score.get("countryCode")
                row["mq_state"]         = score.get("state")
                row["mq_statecode"]     = score.get("stateCode")
                row["mq_county"]        = score.get("county")
                row["mq_countycode"]    = score.get("countyCode")
                row["mq_city"]          = score.get("city")
                row["mq_district"]      = score.get("district")
                row["mq_subdistrict"]   = score.get("subdistrict")
                row["mq_streets"]       = score.get("streets")
                # row["mq_street"]      = _get_first(score.get("streets"))
                row["mq_block"]         = score.get("block")
                row["mq_subblock"]      = score.get("subblock")
                row["mq_postalcode"]    = score.get("postalCode")
                row["mq_housenumber"]   = score.get("houseNumber")
                row["mq_building"]      = score.get("building")
                row["mq_unit"]          = score.get("unit")
                row["mq_placename"]     = score.get("placeName")
                row["mq_ontologyname"]  = score.get("ontologyName")
                # fmt: on

        if not data:
            return GeoDataFrame()

        return GeoDataFrame(data, geometry="position", crs="EPSG:4326")


class _PoiResult(CasaGeoResult):
    """
    Represents the result of a CasaGeoSpatial.poi() query.
    """

    # https://www.here.com/docs/bundle/geocoding-and-search-api-developer-guide/page/topics/endpoint-discover-brief.html
    # https://www.here.com/docs/bundle/geocoding-and-search-api-v7-api-reference/page/index.html#/paths/~1discover/get

    def dataframe(self, id_: Any | None = None) -> GeoDataFrame:
        if id_ is None:
            id_ = str(self._uuid)

        data: list[dict] = []
        for index, item in enumerate(self._data.get("items", [])):
            data.append(row := {})

            # TODO: Address information.

            if True:
                # fmt: off
                row["id"]               = id_
                row["subid"]            = index
                row["title"]            = item.get("title")
                row["hereid"]           = item.get("id")
                row["resulttype"]       = item.get("resultType")
                row["position"]         = _item_position(item)
                row["access"]           = _item_navigation(item)
                row["distance"]         = item.get("distance")
                # fmt: on

        if not data:
            return GeoDataFrame()

        return GeoDataFrame(data, geometry="position", crs="EPSG:4326")


class _RevgeocodeResult(CasaGeoResult):
    """
    Represents the result of a CasaGeoSpatial.revgeocode() query.
    """

    # https://www.here.com/docs/bundle/geocoding-and-search-api-developer-guide/page/topics/endpoint-reverse-geocode-brief.html
    # https://www.here.com/docs/bundle/geocoding-and-search-api-v7-api-reference/page/index.html#/paths/~1revgeocode/get

    def first_title(self) -> str | None:
        try:
            return self._data["items"][0]["title"]
        except (KeyError, IndexError):
            return None

    def dataframe(self, id_: Any | None = None) -> GeoDataFrame:
        if id_ is None:
            id_ = str(self._uuid)

        data: list[dict] = []
        for index, item in enumerate(self._data.get("items", [])):
            data.append(row := {})

            # TODO: Address information.

            if True:
                # fmt: off
                row["id"]               = id_
                row["subid"]            = index
                row["title"]            = item.get("title")
                row["hereid"]           = item.get("id")
                row["resulttype"]       = item.get("resultType")
                row["position"]         = _item_position(item)
                row["access"]           = _item_navigation(item)
                row["distance"]         = item.get("distance")
                row["mapview"]          = _item_mapview(item)
                # fmt: on

        if not data:
            return GeoDataFrame()

        return GeoDataFrame(data, geometry="position", crs="EPSG:4326")


@dataclass
class CasaGeoCoder:
    """
    Provides geocoding operations.

    Attributes:
        client:
            The client object authorizing these queries.

        language:
            The preferred language for the response.

            This must be a valid IETF BCP47 language tag, such as "en-US", or a
            comma-separated list of such tags in order of preference.

        political_view:
            The political view of the query regarding disputed territories.

            This must be a valid ISO 3166-1 alpha-3 country code.

        position:
            The search context position.

        limit:
            Limit on the number of results to return for each query. Must be
            between 1 and 100. ``None`` means to use the API’s default value.

        countries:
            If set, the search is limited to the specified countries. Must be a
            sequence of valid ISO 3166-1 alpha-3 country codes.

        address_names_mode:
            How to handle places with multiple names.

            - ``None``: Prefer matched names for administrative places and
              normalized names for street names.
            - ``"matched"``: Prefer names that match the input query.
            - ``"normalized"``: Prefer the official names of places.

        postal_code_mode:
            How to handle postal codes spanning multiple cities or districts.

            - ``None``: Return only one result per postal code, leaving the city
              or district name blank if necessary.
            - ``"cityLookup"``: When a postal code spans multiple cities, return
              all possible combinations of the postal code with the
              corresponding city names.
            - ``"districtLookup"``: When a postal code spans multiple districts,
              return all possible combinations of the postal code with the
              corresponding city and district names.
    """

    client: CasaGeoClient
    _: KW_ONLY
    language: str | None = None
    political_view: str | None = None
    position: Point | None = None
    limit: int | None = None
    countries: Sequence[str] | None = None
    address_names_mode: str | None = None
    postal_code_mode: str | None = None

    def __post_init__(self) -> None:
        # Check that self.client is not a string, because the beta version of
        # this class took the license key as the first parameter.
        if not isinstance(self.client, CasaGeoClient):
            raise TypeError("client must be a CasaGeoClient instance")
        if self.political_view is None:
            self.political_view = self.client.preferred_political_view

    def _validate(self) -> None:

        if self.language is not None:
            if not self.language:
                raise ValueError("language must not be empty")
            for tag in self.language.split(","):
                _util.validate_ietf_bcp47_language_tag(tag)

        if self.political_view is not None:
            _util.validate_iso3166_alpha3_country_code(self.political_view)

        if self.position is not None:
            if not self.position.is_valid:
                raise ValueError("position must be a valid 2D point")

        if self.limit is not None:
            if not 1 <= self.limit <= 100:
                raise ValueError("limit must be between 1 and 100 inclusive")

        if self.countries is not None:
            if not self.countries:
                raise ValueError("countries must not be empty")
            for ctr in self.countries:
                _util.validate_iso3166_alpha3_country_code(ctr)

        if self.address_names_mode is not None:
            if self.address_names_mode not in _consts.ADDRESS_NAMES_MODES:
                raise ValueError(
                    f"address_names_mode must be one of {_consts.ADDRESS_NAMES_MODES}, got {self.address_names_mode!r}"
                )

        if self.postal_code_mode is not None:
            if self.postal_code_mode not in _consts.POSTAL_CODE_MODES:
                raise ValueError(
                    f"postal_code_mode must be one of {_consts.POSTAL_CODE_MODES}, got {self.postal_code_mode!r}"
                )

    @AddressResult.wrap_errors()
    def address(
        self,
        address: str | None = None,
        *,
        country: str | None = None,
        state: str | None = None,
        county: str | None = None,
        city: str | None = None,
        district: str | None = None,
        street: str | None = None,
        housenumber: str | None = None,
        postalcode: str | None = None,
    ) -> AddressResult:
        """
        Geocode an address and return the API response as an AddressResult object.

        You can supply either a free-form address string or a combination of
        country, state, county, city, district, street, house number and postal
        code. It is currently not possible to supply structured parameters
        and a free-form address string at the same time, but this restriction
        will be lifted in a future version.

        If the input data combines street and house number information into a
        single field, you can pass the resulting string to the ``street``
        parameter and leave the ``housenumber`` parameter unset.

        The structured parameters are not hard filters, meaning it is possible to
        receive results from outside the specified country or city. The
        possibility to add hard filters to queries will be added in a future
        version.

        Args:
            address: A free-form address string.
            country: The name of a country.
            state: The name of a state or province.
            county: The name of a county.
            city: The name of a city.
            district: The name of a city district.
            street: The name of a street.
            housenumber: The house number.
            postalcode: The postal code.

        Returns:
            AddressResult: An AddressResult instance.
        """
        from ._apiclient.geocoder.api.default import get_geocode
        from ._apiclient.geocoder.models import (
            GetGeocodeAddressNamesMode,
            GetGeocodePostalCodeMode,
        )
        from ._apiclient.geocoder.types import UNSET

        def _or_unset(val, /):
            return val if val is not None else UNSET

        self._validate()

        fields = {
            "country": country,
            "state": state,
            "county": county,
            "city": city,
            "district": district,
            "street": street,
            "houseNumber": housenumber,
            "postalCode": postalcode,
        }

        response = get_geocode.sync_detailed(
            client=self.client._geocoding_client,
            q=_or_unset(address),
            qq=_qq(fields) or UNSET,
            at=(
                _util.point_to_latlng(self.position)
                if self.position is not None
                else UNSET
            ),
            in_=(
                ("countryCode:" + ",".join(self.countries))
                if self.countries is not None
                else UNSET
            ),
            # types=(...),
            # with_=(...),
            lang=(self.language.split(",") if self.language is not None else UNSET),
            limit=_or_unset(self.limit),
            political_view=_or_unset(self.political_view),
            address_names_mode=(
                GetGeocodeAddressNamesMode(self.address_names_mode)
                if self.address_names_mode is not None
                else UNSET
            ),
            postal_code_mode=(
                GetGeocodePostalCodeMode(self.postal_code_mode)
                if self.postal_code_mode is not None
                else UNSET
            ),
        )

        return AddressResult.from_response(response)

    # https://www.here.com/docs/bundle/geocoding-and-search-api-developer-guide/page/topics/endpoint-discover-brief.html
    # https://www.here.com/docs/bundle/geocoding-and-search-api-v7-api-reference/page/index.html#/paths/~1discover/get
    @_PoiResult.wrap_errors()
    def _poi(self, position: Point, *, limit: int = 20) -> _PoiResult:
        # TODO: Implement API parameter 'query: str' for a proper discover() method.
        from ._apiclient.old.api.v1 import v1_poi_retrieve

        result = v1_poi_retrieve.sync_detailed(
            client=self.client._v1_client,
            at=_util.point_to_latlng(position),
            limit=limit,
        )
        return _PoiResult.from_response(result)

    # https://www.here.com/docs/bundle/geocoding-and-search-api-developer-guide/page/topics/endpoint-reverse-geocode-brief.html
    # https://www.here.com/docs/bundle/geocoding-and-search-api-v7-api-reference/page/index.html#/paths/~1revgeocode/get
    @_RevgeocodeResult.wrap_errors()
    def _revgeocode(self, position: Point) -> _RevgeocodeResult:
        # TODO: Implement API parameter 'type: Iterable[str] | None = None'.
        # TODO: Implement API parameter 'limit: int = 1'.
        from ._apiclient.old.api.v1 import v1_revgeocode_retrieve
        from ._apiclient.old.types import UNSET

        result = v1_revgeocode_retrieve.sync_detailed(
            client=self.client._v1_client,
            at=_util.point_to_latlng(position),
            lang=(self.language if self.language is not None else UNSET),
        )
        return _RevgeocodeResult.from_response(result)

    def _bulk_address_json(self, address_df: DataFrame) -> dict:
        """
        Geocode all addresses in the input dataframe and return a dict.

        Warning: If a request errors, it may drop the previous data.
        """
        if not (info := self.client.account_info()):
            raise CasaGeoError(
                "Could not retrieve account information for bulk cost calculation"
            ) from info.error()

        estimated_cost = len(address_df) * _consts.GEOCODE_COST
        if info.credits() < estimated_cost:
            raise CasaGeoError(
                f"Insufficient credits to geocode {len(address_df)} addresses"
            )
        out = {}
        for i, row in address_df.iterrows():
            out[i] = self.address(**row.to_dict()).json()
            out[i]["id"] = row["id"]
        return out



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Shape")


@_attrs_define
class Shape:
    """A struct used to control the shape of the returned isolines.

    Attributes:
        max_points (int | Unset): Limits the number of points in the resulting isoline geometry.

            If the isoline consists of multiple components, the sum of points from all components is considered.
            This parameter doesn't affect performance. Look at `optimizeFor` parameter to optimize for performance.

            Notes:
              Quality of isolines degrades as maxPoints value is decreased,
              It is Recommended use `maxPoints` value greater than 100 for optimal quality isolines.
             Example: 150.
        max_resolution (int | Unset): Distance in meters. Default: 0. Example: 189.
    """

    max_points: int | Unset = UNSET
    max_resolution: int | Unset = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        max_points = self.max_points

        max_resolution = self.max_resolution

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if max_points is not UNSET:
            field_dict["maxPoints"] = max_points
        if max_resolution is not UNSET:
            field_dict["maxResolution"] = max_resolution

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        max_points = d.pop("maxPoints", UNSET)

        max_resolution = d.pop("maxResolution", UNSET)

        shape = cls(
            max_points=max_points,
            max_resolution=max_resolution,
        )

        shape.additional_properties = d
        return shape

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.rerouting_mode import ReroutingMode
from ..types import UNSET, Unset

T = TypeVar("T", bound="Rerouting")


@_attrs_define
class Rerouting:
    """Rerouting parameters allow to request a new route calculation based on the route handle.

    All attributes are optional.

        Example:
            {'$ref': '#/components/examples/ReroutingExample'}

        Attributes:
            mode (ReroutingMode | Unset): Defines what kind of additional route calculation should be done.

                * `none` - Cuts off route before the current position. Updates dynamic attributes of the
                route after the current position. If the current position, i.e., the new `origin`, is defined and is
                outside of the original route then the request will fail. If both `origin` and
                `lastTraveledSectionIndex` or `traveledDistanceOnLastSection` are provided then
                `lastTraveledSectionIndex` and `traveledDistanceOnLastSection` will be applied first, and
                then `origin` will be matched to the part of the route that's left. This is the default behavior.
                * `returnToRoute` - Same as `none` if current position is on the route. If the current
                position is not on the original route then a new route to the destination will be
                calculated, starting from the current position. The new route will try to preserve the shape
                of the original route, if possible. If a new optimal route is found before a route back to
                the original route then the new route will be returned.
            last_traveled_section_index (int | Unset): Can be used to indicate the index of the last traveled route section
                on multi-section
                routes. The traveled part of the route won't be reused.

                If the last traveled section is specified and the current position, i.e., the new
                `origin`, is provided, it is expected to be on the last traveled section. If it is
                not on the last traveled section, it is treated as not on the route, triggering
                return to route, if enabled, to the waypoint at the end of the last traveled section.

                If the last traveled section index is not specified, the current position can be
                matched to any section of the route. In this case the route will continue to the
                next waypoint after the current position, i.e., the new `origin`. If the current
                position is not on the route, return to route, if enabled, will by default return
                to the waypoint at the end of the first section.
            traveled_distance_on_last_section (int | Unset): Offset in meter to the last visited position on the route
                section defined by the `lastTraveledSectionIndex`.
                 Default: 0.
    """

    mode: ReroutingMode | Unset = UNSET
    last_traveled_section_index: int | Unset = UNSET
    traveled_distance_on_last_section: int | Unset = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        mode: str | Unset = UNSET
        if not isinstance(self.mode, Unset):
            mode = self.mode.value

        last_traveled_section_index = self.last_traveled_section_index

        traveled_distance_on_last_section = self.traveled_distance_on_last_section

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if mode is not UNSET:
            field_dict["mode"] = mode
        if last_traveled_section_index is not UNSET:
            field_dict["lastTraveledSectionIndex"] = last_traveled_section_index
        if traveled_distance_on_last_section is not UNSET:
            field_dict["traveledDistanceOnLastSection"] = (
                traveled_distance_on_last_section
            )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _mode = d.pop("mode", UNSET)
        mode: ReroutingMode | Unset
        if isinstance(_mode, Unset):
            mode = UNSET
        else:
            mode = ReroutingMode(_mode)

        last_traveled_section_index = d.pop("lastTraveledSectionIndex", UNSET)

        traveled_distance_on_last_section = d.pop(
            "traveledDistanceOnLastSection", UNSET
        )

        rerouting = cls(
            mode=mode,
            last_traveled_section_index=last_traveled_section_index,
            traveled_distance_on_last_section=traveled_distance_on_last_section,
        )

        rerouting.additional_properties = d
        return rerouting

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .access_point_place import AccessPointPlace
from .access_point_place_side_of_street import AccessPointPlaceSideOfStreet
from .access_point_place_type import AccessPointPlaceType
from .agency import Agency
from .allow import Allow
from .arrive_action import ArriveAction
from .arrive_action_action import ArriveActionAction
from .attribution import Attribution
from .attribution_link_type import AttributionLinkType
from .auth_error_response_schema import AuthErrorResponseSchema
from .avoid import Avoid
from .avoid_post import AvoidPost
from .axle_group_weight import AxleGroupWeight
from .base_action import BaseAction
from .base_notice_detail import BaseNoticeDetail
from .base_summary import BaseSummary
from .board_action import BoardAction
from .board_action_action import BoardActionAction
from .bounding_box_area import BoundingBoxArea
from .bounding_box_area_with_exceptions import BoundingBoxAreaWithExceptions
from .calculate_routes_post_parameters import CalculateRoutesPostParameters
from .charge_point_operator import ChargePointOperator
from .charging_action import ChargingAction
from .charging_action_action import ChargingActionAction
from .charging_connector_attributes import ChargingConnectorAttributes
from .charging_setup_action import ChargingSetupAction
from .charging_setup_action_action import ChargingSetupActionAction
from .charging_station_brand import ChargingStationBrand
from .charging_station_place import ChargingStationPlace
from .charging_station_place_side_of_street import ChargingStationPlaceSideOfStreet
from .charging_station_place_type import ChargingStationPlaceType
from .continue_action import ContinueAction
from .continue_action_action import ContinueActionAction
from .coordinate import Coordinate
from .corridor_area import CorridorArea
from .corridor_area_with_exceptions import CorridorAreaWithExceptions
from .data_version import DataVersion
from .deboard_action import DeboardAction
from .deboard_action_action import DeboardActionAction
from .depart_action import DepartAction
from .depart_action_action import DepartActionAction
from .docking_station_place import DockingStationPlace
from .docking_station_place_side_of_street import DockingStationPlaceSideOfStreet
from .docking_station_place_type import DockingStationPlaceType
from .driver import Driver
from .dynamic_speed_info import DynamicSpeedInfo
from .e_mobility_service_provider import EMobilityServiceProvider
from .encoded_corridor_area import EncodedCorridorArea
from .encoded_corridor_area_with_exceptions import EncodedCorridorAreaWithExceptions
from .encoded_polygon_area import EncodedPolygonArea
from .encoded_polygon_area_with_exceptions import EncodedPolygonAreaWithExceptions
from .error_response import ErrorResponse
from .ev_empirical_model import EVEmpiricalModel
from .ev_physical_model import EVPhysicalModel
from .ev_post import EVPost
from .exclude import Exclude
from .exclude_post import ExcludePost
from .exit_action import ExitAction
from .exit_action_action import ExitActionAction
from .exit_info import ExitInfo
from .fare import Fare
from .fare_pass import FarePass
from .fare_pass_validity_period import FarePassValidityPeriod
from .fuel import Fuel
from .get_routes_by_handle_post_parameters import GetRoutesByHandlePostParameters
from .hazardous_goods_restriction_any import HazardousGoodsRestrictionAny
from .health_response_fail_schema import HealthResponseFailSchema
from .health_response_fail_schema_status import HealthResponseFailSchemaStatus
from .health_response_ok_schema import HealthResponseOKSchema
from .health_response_ok_schema_status import HealthResponseOKSchemaStatus
from .license_plate_restriction import LicensePlateRestriction
from .localized_route_number import LocalizedRouteNumber
from .localized_route_number_direction import LocalizedRouteNumberDirection
from .localized_string import LocalizedString
from .location import Location
from .match_trace import MatchTrace
from .match_trace_point import MatchTracePoint
from .match_trace_via import MatchTraceVia
from .max_speed_on_segment_post_inner import MaxSpeedOnSegmentPostInner
from .max_speed_type_1 import MaxSpeedType1
from .notice import Notice
from .notice_severity import NoticeSeverity
from .offset_action import OffsetAction
from .parking_lot_place import ParkingLotPlace
from .parking_lot_place_side_of_street import ParkingLotPlaceSideOfStreet
from .parking_lot_place_type import ParkingLotPlaceType
from .passthrough import Passthrough
from .pedestrian_departure import PedestrianDeparture
from .pedestrian_notice import PedestrianNotice
from .pedestrian_section import PedestrianSection
from .pedestrian_section_type import PedestrianSectionType
from .pedestrian_span import PedestrianSpan
from .pedestrian_summary import PedestrianSummary
from .pedestrian_transport import PedestrianTransport
from .place import Place
from .place_side_of_street import PlaceSideOfStreet
from .place_type import PlaceType
from .polygon_area import PolygonArea
from .polygon_area_with_exceptions import PolygonAreaWithExceptions
from .range_price import RangePrice
from .ref_replacements import RefReplacements
from .rerouting import Rerouting
from .rerouting_mode import ReroutingMode
from .return_ import Return
from .road_info import RoadInfo
from .road_info_type import RoadInfoType
from .route_label import RouteLabel
from .route_label_label_type import RouteLabelLabelType
from .route_response_notice import RouteResponseNotice
from .router_mode import RouterMode
from .router_route import RouterRoute
from .router_route_response import RouterRouteResponse
from .routing_error_response import RoutingErrorResponse
from .routing_mode import RoutingMode
from .routing_zone import RoutingZone
from .scooter import Scooter
from .signpost_info import SignpostInfo
from .signpost_label_route_number import SignpostLabelRouteNumber
from .signpost_label_text import SignpostLabelText
from .simple_turn_action import SimpleTurnAction
from .simple_turn_action_action import SimpleTurnActionAction
from .single_price import SinglePrice
from .spans import Spans
from .station_place import StationPlace
from .station_place_side_of_street import StationPlaceSideOfStreet
from .station_place_type import StationPlaceType
from .taxi import Taxi
from .time_restricted_price import TimeRestrictedPrice
from .time_restricted_weekdays import TimeRestrictedWeekdays
from .toll_collection_location import TollCollectionLocation
from .toll_cost import TollCost
from .toll_country_summary import TollCountrySummary
from .toll_fare import TollFare
from .toll_summary import TollSummary
from .toll_system import TollSystem
from .toll_system_summary import TollSystemSummary
from .tolls import Tolls
from .traffic import Traffic
from .traffic_incident import TrafficIncident
from .traffic_incident_criticality import TrafficIncidentCriticality
from .traffic_mode import TrafficMode
from .trailer_count_range import TrailerCountRange
from .transit_departure import TransitDeparture
from .transit_incident import TransitIncident
from .transit_notice import TransitNotice
from .transit_section import TransitSection
from .transit_section_type import TransitSectionType
from .transit_span import TransitSpan
from .transit_stop import TransitStop
from .transit_transport import TransitTransport
from .transponder_system import TransponderSystem
from .truck import Truck
from .truck_axle_count_range import TruckAxleCountRange
from .truck_category import TruckCategory
from .truck_engine_type import TruckEngineType
from .truck_type import TruckType
from .truck_type_with_default import TruckTypeWithDefault
from .tunnel_category import TunnelCategory
from .turn_action import TurnAction
from .turn_action_action import TurnActionAction
from .turn_action_direction import TurnActionDirection
from .turn_action_severity import TurnActionSeverity
from .units import Units
from .vehicle import Vehicle
from .vehicle_category import VehicleCategory
from .vehicle_departure import VehicleDeparture
from .vehicle_engine_type import VehicleEngineType
from .vehicle_notice import VehicleNotice
from .vehicle_restriction import VehicleRestriction
from .vehicle_restriction_max_weight import VehicleRestrictionMaxWeight
from .vehicle_restriction_type import VehicleRestrictionType
from .vehicle_section import VehicleSection
from .vehicle_section_type import VehicleSectionType
from .vehicle_span import VehicleSpan
from .vehicle_summary import VehicleSummary
from .vehicle_transport import VehicleTransport
from .vehicle_travel_summary import VehicleTravelSummary
from .vehicle_type import VehicleType
from .version_response import VersionResponse
from .via_notice_detail import ViaNoticeDetail
from .via_notice_detail_type import ViaNoticeDetailType
from .violated_charging_station_opening_hours import ViolatedChargingStationOpeningHours
from .violated_charging_station_opening_hours_type import (
    ViolatedChargingStationOpeningHoursType,
)
from .violated_transport_mode import ViolatedTransportMode
from .violated_transport_mode_type import ViolatedTransportModeType
from .violated_truck_road_type import ViolatedTruckRoadType
from .violated_truck_road_type_type import ViolatedTruckRoadTypeType
from .violated_zone_reference import ViolatedZoneReference
from .violated_zone_reference_type import ViolatedZoneReferenceType
from .wait_action import WaitAction
from .wait_action_action import WaitActionAction
from .web_link import WebLink
from .web_link_with_device_type import WebLinkWithDeviceType
from .wheelchair_accessibility import WheelchairAccessibility

__all__ = (
    "AccessPointPlace",
    "AccessPointPlaceSideOfStreet",
    "AccessPointPlaceType",
    "Agency",
    "Allow",
    "ArriveAction",
    "ArriveActionAction",
    "Attribution",
    "AttributionLinkType",
    "AuthErrorResponseSchema",
    "Avoid",
    "AvoidPost",
    "AxleGroupWeight",
    "BaseAction",
    "BaseNoticeDetail",
    "BaseSummary",
    "BoardAction",
    "BoardActionAction",
    "BoundingBoxArea",
    "BoundingBoxAreaWithExceptions",
    "CalculateRoutesPostParameters",
    "ChargePointOperator",
    "ChargingAction",
    "ChargingActionAction",
    "ChargingConnectorAttributes",
    "ChargingSetupAction",
    "ChargingSetupActionAction",
    "ChargingStationBrand",
    "ChargingStationPlace",
    "ChargingStationPlaceSideOfStreet",
    "ChargingStationPlaceType",
    "ContinueAction",
    "ContinueActionAction",
    "Coordinate",
    "CorridorArea",
    "CorridorAreaWithExceptions",
    "DataVersion",
    "DeboardAction",
    "DeboardActionAction",
    "DepartAction",
    "DepartActionAction",
    "DockingStationPlace",
    "DockingStationPlaceSideOfStreet",
    "DockingStationPlaceType",
    "Driver",
    "DynamicSpeedInfo",
    "EMobilityServiceProvider",
    "EncodedCorridorArea",
    "EncodedCorridorAreaWithExceptions",
    "EncodedPolygonArea",
    "EncodedPolygonAreaWithExceptions",
    "ErrorResponse",
    "EVEmpiricalModel",
    "EVPhysicalModel",
    "EVPost",
    "Exclude",
    "ExcludePost",
    "ExitAction",
    "ExitActionAction",
    "ExitInfo",
    "Fare",
    "FarePass",
    "FarePassValidityPeriod",
    "Fuel",
    "GetRoutesByHandlePostParameters",
    "HazardousGoodsRestrictionAny",
    "HealthResponseFailSchema",
    "HealthResponseFailSchemaStatus",
    "HealthResponseOKSchema",
    "HealthResponseOKSchemaStatus",
    "LicensePlateRestriction",
    "LocalizedRouteNumber",
    "LocalizedRouteNumberDirection",
    "LocalizedString",
    "Location",
    "MatchTrace",
    "MatchTracePoint",
    "MatchTraceVia",
    "MaxSpeedOnSegmentPostInner",
    "MaxSpeedType1",
    "Notice",
    "NoticeSeverity",
    "OffsetAction",
    "ParkingLotPlace",
    "ParkingLotPlaceSideOfStreet",
    "ParkingLotPlaceType",
    "Passthrough",
    "PedestrianDeparture",
    "PedestrianNotice",
    "PedestrianSection",
    "PedestrianSectionType",
    "PedestrianSpan",
    "PedestrianSummary",
    "PedestrianTransport",
    "Place",
    "PlaceSideOfStreet",
    "PlaceType",
    "PolygonArea",
    "PolygonAreaWithExceptions",
    "RangePrice",
    "RefReplacements",
    "Rerouting",
    "ReroutingMode",
    "Return",
    "RoadInfo",
    "RoadInfoType",
    "RouteLabel",
    "RouteLabelLabelType",
    "RouteResponseNotice",
    "RouterMode",
    "RouterRoute",
    "RouterRouteResponse",
    "RoutingErrorResponse",
    "RoutingMode",
    "RoutingZone",
    "Scooter",
    "SignpostInfo",
    "SignpostLabelRouteNumber",
    "SignpostLabelText",
    "SimpleTurnAction",
    "SimpleTurnActionAction",
    "SinglePrice",
    "Spans",
    "StationPlace",
    "StationPlaceSideOfStreet",
    "StationPlaceType",
    "Taxi",
    "TimeRestrictedPrice",
    "TimeRestrictedWeekdays",
    "TollCollectionLocation",
    "TollCost",
    "TollCountrySummary",
    "TollFare",
    "Tolls",
    "TollSummary",
    "TollSystem",
    "TollSystemSummary",
    "Traffic",
    "TrafficIncident",
    "TrafficIncidentCriticality",
    "TrafficMode",
    "TrailerCountRange",
    "TransitDeparture",
    "TransitIncident",
    "TransitNotice",
    "TransitSection",
    "TransitSectionType",
    "TransitSpan",
    "TransitStop",
    "TransitTransport",
    "TransponderSystem",
    "Truck",
    "TruckAxleCountRange",
    "TruckCategory",
    "TruckEngineType",
    "TruckType",
    "TruckTypeWithDefault",
    "TunnelCategory",
    "TurnAction",
    "TurnActionAction",
    "TurnActionDirection",
    "TurnActionSeverity",
    "Units",
    "Vehicle",
    "VehicleCategory",
    "VehicleDeparture",
    "VehicleEngineType",
    "VehicleNotice",
    "VehicleRestriction",
    "VehicleRestrictionMaxWeight",
    "VehicleRestrictionType",
    "VehicleSection",
    "VehicleSectionType",
    "VehicleSpan",
    "VehicleSummary",
    "VehicleTransport",
    "VehicleTravelSummary",
    "VehicleType",
    "VersionResponse",
    "ViaNoticeDetail",
    "ViaNoticeDetailType",
    "ViolatedChargingStationOpeningHours",
    "ViolatedChargingStationOpeningHoursType",
    "ViolatedTransportMode",
    "ViolatedTransportModeType",
    "ViolatedTruckRoadType",
    "ViolatedTruckRoadTypeType",
    "ViolatedZoneReference",
    "ViolatedZoneReferenceType",
    "WaitAction",
    "WaitActionAction",
    "WebLink",
    "WebLinkWithDeviceType",
    "WheelchairAccessibility",
)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="Connection")


@_attrs_define
class Connection:
    """Connection represents the geometry of special links that were reached but not included in the components.
    These links are connections like ferries.

        Attributes:
            from_polygon_index (int): Index of start component of the connection
            to_polygon_index (int): Index of end component of the connection
            polyline (str): Line string in [Flexible Polyline](https://github.com/heremaps/flexible-polyline) format.
                Coordinates are in the WGS84 coordinate system, including `Elevation` (if present). Example:
                A05xgKuy2xCx9B7vUl0OhnR54EqSzpEl-HxjD3pBiGnyGi2CvwFsgD3nD4vB6e.
    """

    from_polygon_index: int
    to_polygon_index: int
    polyline: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from_polygon_index = self.from_polygon_index

        to_polygon_index = self.to_polygon_index

        polyline = self.polyline

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "fromPolygonIndex": from_polygon_index,
            "toPolygonIndex": to_polygon_index,
            "polyline": polyline,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        from_polygon_index = d.pop("fromPolygonIndex")

        to_polygon_index = d.pop("toPolygonIndex")

        polyline = d.pop("polyline")

        connection = cls(
            from_polygon_index=from_polygon_index,
            to_polygon_index=to_polygon_index,
            polyline=polyline,
        )

        connection.additional_properties = d
        return connection

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.route_label_label_type import RouteLabelLabelType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.localized_string import LocalizedString


T = TypeVar("T", bound="RouteLabel")


@_attrs_define
class RouteLabel:
    """Road name or route number distinguishing a route from other alternatives.

    Attributes:
        label_type (RouteLabelLabelType | Unset):
        name (LocalizedString | Unset): String with optional language code. Example: {'value': 'Invalidenstraße',
            'language': 'de'}.
    """

    label_type: RouteLabelLabelType | Unset = UNSET
    name: LocalizedString | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        label_type: str | Unset = UNSET
        if not isinstance(self.label_type, Unset):
            label_type = self.label_type.value

        name: dict[str, Any] | Unset = UNSET
        if not isinstance(self.name, Unset):
            name = self.name.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if label_type is not UNSET:
            field_dict["label_type"] = label_type
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.localized_string import LocalizedString

        d = dict(src_dict)
        _label_type = d.pop("label_type", UNSET)
        label_type: RouteLabelLabelType | Unset
        if isinstance(_label_type, Unset):
            label_type = UNSET
        else:
            label_type = RouteLabelLabelType(_label_type)

        _name = d.pop("name", UNSET)
        name: LocalizedString | Unset
        if isinstance(_name, Unset):
            name = UNSET
        else:
            name = LocalizedString.from_dict(_name)

        route_label = cls(
            label_type=label_type,
            name=name,
        )

        route_label.additional_properties = d
        return route_label

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.data_version import DataVersion


T = TypeVar("T", bound="VersionResponse")


@_attrs_define
class VersionResponse:
    """Returns the versions of the service components.

    Attributes:
        api_version (str): The current version of the API. Example: 8.18.0.
        service_version (str | Unset): The current version of the service. Example: 2022-12-15-b706cc8c-9057409.
        data_versions (list[DataVersion] | Unset): Returns the versions of data sets used by the service.
    """

    api_version: str
    service_version: str | Unset = UNSET
    data_versions: list[DataVersion] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        api_version = self.api_version

        service_version = self.service_version

        data_versions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.data_versions, Unset):
            data_versions = []
            for data_versions_item_data in self.data_versions:
                data_versions_item = data_versions_item_data.to_dict()
                data_versions.append(data_versions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "apiVersion": api_version,
        })
        if service_version is not UNSET:
            field_dict["serviceVersion"] = service_version
        if data_versions is not UNSET:
            field_dict["dataVersions"] = data_versions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.data_version import DataVersion

        d = dict(src_dict)
        api_version = d.pop("apiVersion")

        service_version = d.pop("serviceVersion", UNSET)

        _data_versions = d.pop("dataVersions", UNSET)
        data_versions: list[DataVersion] | Unset = UNSET
        if _data_versions is not UNSET:
            data_versions = []
            for data_versions_item_data in _data_versions:
                data_versions_item = DataVersion.from_dict(data_versions_item_data)

                data_versions.append(data_versions_item)

        version_response = cls(
            api_version=api_version,
            service_version=service_version,
            data_versions=data_versions,
        )

        version_response.additional_properties = d
        return version_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.ev_station_connector_type_ids_item import EvStationConnectorTypeIdsItem
from ..models.ev_station_current import EvStationCurrent
from ..models.ev_station_payment_method_ids_item import EvStationPaymentMethodIdsItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="EvStation")


@_attrs_define
class EvStation:
    """
    Attributes:
        payment_method_ids (list[EvStationPaymentMethodIdsItem] | Unset): Filter to retrieve Electric Vehicle charging
            stations by supported payment types

            Description of supported values:

            - `authByCarPlugAndCharge`: indicates whether this EV station supports Plug&Charge.ISO 15118 Plug&Charge enables
            drivers to plug in and charge up instantly using automatic EV-to-charging station authentication technology.
            - `creditCard`: indicates if this EV station accepts credit card payment or not.
            - `debitCard`: indicates if this EV station accepts debit card payment or not.
            - `onlineApplePay`: indicates whether this EV station accepts Apple Pay (TM) authentication and payment method.
            - `onlineGooglePay`:  indicates whether this EV station accepts Google Pay (TM) authentication and payment
            method.
            - `onlinePaypal`: indicates whether this EV station accepts PayPal (TM) authentication and payment method.
            - `operatorApp`: indicates whether this EV station support authentication and payment through an app from the
            Charging Point Operator.
        e_mobility_service_provider_partner_ids (list[str] | Unset): Filter to retrieve Electric Vehicle charging
            stations by value of e-Mobility Service Providers
            (eMSP) partner ID. EV roaming enables EV drivers to be charged using subscription cards from eMSP.
            Several partner IDs can be set as value this filter, separated with a comma.
            The list of eMSP partner IDs supported by this filter can be retrieved using the EV Charge Points API
            [`roaming`](https://www.here.com/docs/bundle/ev-charge-points-api-developer-guide/page/topics/resource-
            roamings.html)
            resource. Example: ['1f351b84-cca5-11ed-8155-42010aa40002', '1f39ad02-cca5-11ed-9d1a-42010aa40002'].
        current (EvStationCurrent | Unset): Filter to retrieve Electric Vehicle charging stations by current type

            Description of supported values:

            - `AC`: Alternating current
            - `DC`: Direct current Example: AC.
        supplier_names (list[str] | Unset): Filter to retrieve Electric Vehicle charging stations with at least one
            connector of supplier name in the filter
        connector_type_ids (list[EvStationConnectorTypeIdsItem] | Unset): Filter to retrieve Electric Vehicle charging
            stations with at least a connector type ID

            Description of supported values:

            - `10`: Domestic plug/socket type E+F (CEE 7/7)
            - `11`: Domestic plug/socket type G (BS 1363, IS 401 & 411, MS 58)
            - `12`: Domestic plug/socket type H (SI 32)
            - `13`: Domestic plug/socket type I (AS/NZS 3112)
            - `14`: Domestic plug/socket type I (CPCS-CCC)
            - `15`: Domestic plug/socket type I (IRAM 2073)
            - `20`: Domestic plug/socket type K (Section 107-2-D1)
            - `21`: Domestic plug/socket type K (Thailand TIS 166 - 2549)
            - `22`: Domestic plug/socket type L (CEI 23-16/VII)
            - `23`: Domestic plug/socket type M (South African 15 A/250 V)
            - `24`: Domestic plug/socket type IEC 60906-1 (3 pin)
            - `25`: AVCON Connector
            - `29`: JEVS G 105 (CHAdeMO)
            - `30`: IEC 62196-2 type 1 (SAE J1772)
            - `31`: IEC 62196-2 type 2 (Mennekes)
            - `32`: IEC 62196-2 type 3c (SCAME)
            - `33`: IEC 62196-3 type 1 combo (SAE J1772)
            - `34`: IEC 62196-3 type 2 combo (Mennekes)
            - `35`: IEC 60309 : industrial P + N + E (AC)
            - `36`: IEC 60309 : industrial 3P + E + N (AC)
            - `37`: IEC 60309 : industrial 2P + E (AC)
            - `42`: Domestic plug/socket type J (SEV 1011) (T13, T23)
            - `43`: Tesla Connector
            - `46`: IEC 60309 : industrial 2P + E (DC)
            - `48`: Domestic plug/socket type A (NEMA 1-15, 2 pins)
            - `49`: Domestic plug/socket type C (CEE 7/17, 2 pins)
            - `5`: Domestic plug/socket type B (NEMA 5-15)
            - `50`: IEC 62196-2 type 3a (SCAME)
            - `52`: GB/T (Chinese) AC connector
            - `53`: GB/T (Chinese) DC connector
            - `6`: Domestic plug/socket type B (NEMA 5-20)
            - `7`: Domestic plug/socket type D (BS 546 (3 pin))
            - `8`: Domestic plug/socket type E (CEE 7/5)
            - `9`: Domestic plug/socket type F (CEE 7/4 (Schuko))
        min_power (float | Unset): Minimum charging power in KW that each returned EV station must support in at least
            one of its supply element Example: 6.6.
    """

    payment_method_ids: list[EvStationPaymentMethodIdsItem] | Unset = UNSET
    e_mobility_service_provider_partner_ids: list[str] | Unset = UNSET
    current: EvStationCurrent | Unset = UNSET
    supplier_names: list[str] | Unset = UNSET
    connector_type_ids: list[EvStationConnectorTypeIdsItem] | Unset = UNSET
    min_power: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        payment_method_ids: list[str] | Unset = UNSET
        if not isinstance(self.payment_method_ids, Unset):
            payment_method_ids = []
            for payment_method_ids_item_data in self.payment_method_ids:
                payment_method_ids_item = payment_method_ids_item_data.value
                payment_method_ids.append(payment_method_ids_item)

        e_mobility_service_provider_partner_ids: list[str] | Unset = UNSET
        if not isinstance(self.e_mobility_service_provider_partner_ids, Unset):
            e_mobility_service_provider_partner_ids = (
                self.e_mobility_service_provider_partner_ids
            )

        current: str | Unset = UNSET
        if not isinstance(self.current, Unset):
            current = self.current.value

        supplier_names: list[str] | Unset = UNSET
        if not isinstance(self.supplier_names, Unset):
            supplier_names = self.supplier_names

        connector_type_ids: list[str] | Unset = UNSET
        if not isinstance(self.connector_type_ids, Unset):
            connector_type_ids = []
            for connector_type_ids_item_data in self.connector_type_ids:
                connector_type_ids_item = connector_type_ids_item_data.value
                connector_type_ids.append(connector_type_ids_item)

        min_power = self.min_power

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if payment_method_ids is not UNSET:
            field_dict["paymentMethodIds"] = payment_method_ids
        if e_mobility_service_provider_partner_ids is not UNSET:
            field_dict["eMobilityServiceProviderPartnerIds"] = (
                e_mobility_service_provider_partner_ids
            )
        if current is not UNSET:
            field_dict["current"] = current
        if supplier_names is not UNSET:
            field_dict["supplierNames"] = supplier_names
        if connector_type_ids is not UNSET:
            field_dict["connectorTypeIds"] = connector_type_ids
        if min_power is not UNSET:
            field_dict["minPower"] = min_power

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _payment_method_ids = d.pop("paymentMethodIds", UNSET)
        payment_method_ids: list[EvStationPaymentMethodIdsItem] | Unset = UNSET
        if _payment_method_ids is not UNSET:
            payment_method_ids = []
            for payment_method_ids_item_data in _payment_method_ids:
                payment_method_ids_item = EvStationPaymentMethodIdsItem(
                    payment_method_ids_item_data
                )

                payment_method_ids.append(payment_method_ids_item)

        e_mobility_service_provider_partner_ids = cast(
            list[str], d.pop("eMobilityServiceProviderPartnerIds", UNSET)
        )

        _current = d.pop("current", UNSET)
        current: EvStationCurrent | Unset
        if isinstance(_current, Unset):
            current = UNSET
        else:
            current = EvStationCurrent(_current)

        supplier_names = cast(list[str], d.pop("supplierNames", UNSET))

        _connector_type_ids = d.pop("connectorTypeIds", UNSET)
        connector_type_ids: list[EvStationConnectorTypeIdsItem] | Unset = UNSET
        if _connector_type_ids is not UNSET:
            connector_type_ids = []
            for connector_type_ids_item_data in _connector_type_ids:
                connector_type_ids_item = EvStationConnectorTypeIdsItem(
                    connector_type_ids_item_data
                )

                connector_type_ids.append(connector_type_ids_item)

        min_power = d.pop("minPower", UNSET)

        ev_station = cls(
            payment_method_ids=payment_method_ids,
            e_mobility_service_provider_partner_ids=e_mobility_service_provider_partner_ids,
            current=current,
            supplier_names=supplier_names,
            connector_type_ids=connector_type_ids,
            min_power=min_power,
        )

        ev_station.additional_properties = d
        return ev_station

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Fuel")


@_attrs_define
class Fuel:
    """**Disclaimer: This parameter is currently in beta release, and is therefore subject to breaking changes.**

    Fuel parameters to be used for calculating consumption and related CO2 emission, and toll calculation.

    The following attributes are required for calculating consumption:
      * `type`
      * `freeFlowSpeedTable`

    The following attribute is needed for fuel specific toll calculation (if not provided default toll instead of fuel
    specific toll will be returned):
      * `type`

    **Notes**:
    * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are not supported for
    combination of `ev` and `fuel` vehicles.
    * Fuel parameters are not supported in combination with pedestrian and bicycle `transportMode`.

        Attributes:
            type_ (str): Extensible enum: `diesel` `petrol` `lpg` `cng` `lng` `ethanol` `propane` `hydrogen` `...`
                Vehicle fuel type. It is mandatory for calculation of consumption and CO2 emission.
            free_flow_speed_table (str | Unset): Function curve specifying consumption rate at a given speed.

                The format of the string is a comma-separated list of numbers, as follows:

                ```
                <SPEED_0>,<CONSUMPTION_0>,<SPEED_1>,<CONSUMPTION_1>,...,<SPEED_N>,<CONSUMPTION_N>
                ```

                where speed values are strictly increasing, non-negative integers in units of (km/h), and
                consumption values are non-negative floating point values.

                * Unit for EV:
                | Vehicle Type | Unit |
                |--------------|------|
                | Electric | Wh/m i.e., Watt-hours per meter|

                * Units for Fuel-based vehicles:
                | Vehicle Type | Unit |
                |--------------|------|
                | Diesel, Petrol & LPG | ml/m i.e., milliliters per meter|
                | CNG | gm/m i.e., grams per meter|

                The function is linearly interpolated between data points. For speeds less than `SPEED_0`
                the value of the function is `CONSUMPTION_0`, and for speeds greater than `SPEED_N` the
                value of the function is `CONSUMPTION_N`.
                 Example:
                0,0.2394,14,0.2394,36,0.2586,52,0.196,68,0.2074,83,0.238,95,0.2597,105,0.2597,115,0.2964,125,0.3367,135,0.3508.
            traffic_speed_table (str | Unset): Function curve specifying consumption rate at a given speed.

                The format of the string is a comma-separated list of numbers, as follows:

                ```
                <SPEED_0>,<CONSUMPTION_0>,<SPEED_1>,<CONSUMPTION_1>,...,<SPEED_N>,<CONSUMPTION_N>
                ```

                where speed values are strictly increasing, non-negative integers in units of (km/h), and
                consumption values are non-negative floating point values.

                * Unit for EV:
                | Vehicle Type | Unit |
                |--------------|------|
                | Electric | Wh/m i.e., Watt-hours per meter|

                * Units for Fuel-based vehicles:
                | Vehicle Type | Unit |
                |--------------|------|
                | Diesel, Petrol & LPG | ml/m i.e., milliliters per meter|
                | CNG | gm/m i.e., grams per meter|

                The function is linearly interpolated between data points. For speeds less than `SPEED_0`
                the value of the function is `CONSUMPTION_0`, and for speeds greater than `SPEED_N` the
                value of the function is `CONSUMPTION_N`.
                 Example:
                0,0.2394,14,0.2394,36,0.2586,52,0.196,68,0.2074,83,0.238,95,0.2597,105,0.2597,115,0.2964,125,0.3367,135,0.3508.
            additional_consumption (float | Unset): Rate of fuel (in ml/s for diesel, petrol & LPG, gm/s for CNG) consumed
                by the vehicle for any other reason
                additional to fuel consumption for speed.
            ascent (float | Unset): Rate of fuel consumed per meter rise in elevation (in ml/m for diesel, petrol & LPG,
                gm/m for CNG).
    """

    type_: str
    free_flow_speed_table: str | Unset = UNSET
    traffic_speed_table: str | Unset = UNSET
    additional_consumption: float | Unset = UNSET
    ascent: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        free_flow_speed_table = self.free_flow_speed_table

        traffic_speed_table = self.traffic_speed_table

        additional_consumption = self.additional_consumption

        ascent = self.ascent

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
        })
        if free_flow_speed_table is not UNSET:
            field_dict["freeFlowSpeedTable"] = free_flow_speed_table
        if traffic_speed_table is not UNSET:
            field_dict["trafficSpeedTable"] = traffic_speed_table
        if additional_consumption is not UNSET:
            field_dict["additionalConsumption"] = additional_consumption
        if ascent is not UNSET:
            field_dict["ascent"] = ascent

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = d.pop("type")

        free_flow_speed_table = d.pop("freeFlowSpeedTable", UNSET)

        traffic_speed_table = d.pop("trafficSpeedTable", UNSET)

        additional_consumption = d.pop("additionalConsumption", UNSET)

        ascent = d.pop("ascent", UNSET)

        fuel = cls(
            type_=type_,
            free_flow_speed_table=free_flow_speed_table,
            traffic_speed_table=traffic_speed_table,
            additional_consumption=additional_consumption,
            ascent=ascent,
        )

        fuel.additional_properties = d
        return fuel

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.traffic_incident_criticality import TrafficIncidentCriticality
from ..types import UNSET, Unset

T = TypeVar("T", bound="TrafficIncident")


@_attrs_define
class TrafficIncident:
    """An incident describes a temporary event on the road network.
    It typically refers to a real world incident (accident, road construction, weather condition, etc.)
    on a street or street segment

        Attributes:
            description (str | Unset): A human readable description of the incident Example: closed due to roadworks.
            type_ (str | Unset): Extensible enum: `accident` `congestion` `construction` `disabledVehicle` `massTransit`
                `plannedEvent` `roadHazard` `roadClosure` `weather` `laneRestriction` `other` `...`
                An open list of possible incident causes / types.
                Note: Since new types are expected to appear, it is important to check for unknown types when parsing this
                value.
            criticality (TrafficIncidentCriticality | Unset): Describes the impact an incident has on the route.
                * critical - The part of the route the incident affects is not usable.
                * major - Major impact on duration, e.g. stop and go
                * minor - Minor impact on duration, e.g. traffic jam
                * low - Very little impact on duration, e.g. slightly increased traffic
            valid_from (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either `date-time` or `date-
                only` 'T' `partial-time` (ie no time-offset).
            valid_until (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either `date-time` or `date-
                only` 'T' `partial-time` (ie no time-offset).
            id (str | Unset): Traffic Incident unique identifier,

                Example of an incident identifier in standard representation:
                here:traffic:incident:1000155780078589348

                Id usage:
                Incident details can be queried from the traffic service. See
                [this tutorial](https://www.here.com/docs/bundle/traffic-api-developer-guide-v7/page/topics/use-cases/incidents-
                by-id.html).

                **Notice**:
                In most cases, the ID comes from a third party incident supplier.
                This means that once an incident has expired, the ID might be reused
    """

    description: str | Unset = UNSET
    type_: str | Unset = UNSET
    criticality: TrafficIncidentCriticality | Unset = UNSET
    valid_from: datetime.datetime | Unset = UNSET
    valid_until: datetime.datetime | Unset = UNSET
    id: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        description = self.description

        type_ = self.type_

        criticality: str | Unset = UNSET
        if not isinstance(self.criticality, Unset):
            criticality = self.criticality.value

        valid_from: str | Unset = UNSET
        if not isinstance(self.valid_from, Unset):
            valid_from = self.valid_from.isoformat()

        valid_until: str | Unset = UNSET
        if not isinstance(self.valid_until, Unset):
            valid_until = self.valid_until.isoformat()

        id = self.id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if description is not UNSET:
            field_dict["description"] = description
        if type_ is not UNSET:
            field_dict["type"] = type_
        if criticality is not UNSET:
            field_dict["criticality"] = criticality
        if valid_from is not UNSET:
            field_dict["validFrom"] = valid_from
        if valid_until is not UNSET:
            field_dict["validUntil"] = valid_until
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        description = d.pop("description", UNSET)

        type_ = d.pop("type", UNSET)

        _criticality = d.pop("criticality", UNSET)
        criticality: TrafficIncidentCriticality | Unset
        if isinstance(_criticality, Unset):
            criticality = UNSET
        else:
            criticality = TrafficIncidentCriticality(_criticality)

        _valid_from = d.pop("validFrom", UNSET)
        valid_from: datetime.datetime | Unset
        if isinstance(_valid_from, Unset):
            valid_from = UNSET
        else:
            valid_from = isoparse(_valid_from)

        _valid_until = d.pop("validUntil", UNSET)
        valid_until: datetime.datetime | Unset
        if isinstance(_valid_until, Unset):
            valid_until = UNSET
        else:
            valid_until = isoparse(_valid_until)

        id = d.pop("id", UNSET)

        traffic_incident = cls(
            description=description,
            type_=type_,
            criticality=criticality,
            valid_from=valid_from,
            valid_until=valid_until,
            id=id,
        )

        traffic_incident.additional_properties = d
        return traffic_incident

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TruckFuelType(str, Enum):
    TRUCK_CNG = "truck_cng"
    TRUCK_DIESEL = "truck_diesel"
    TRUCK_HYDROGEN = "truck_hydrogen"
    TRUCK_LNG = "truck_lng"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.ev_charging_point import EvChargingPoint
    from ..models.ev_name_id import EvNameId


T = TypeVar("T", bound="EvConnector")


@_attrs_define
class EvConnector:
    """
    Attributes:
        supplier_name (str | Unset): The EV charge point operator
        connector_type (EvNameId | Unset):
        fixed_cable (bool | Unset): Boolean indicating if a cable is provided for the connector group.If true, then
            there is a cable for the connector group at the station.
        max_power_level (float | Unset): Maximum charge power (in kilowatt) of connectors in connectors group.
        charging_point (EvChargingPoint | Unset):
    """

    supplier_name: str | Unset = UNSET
    connector_type: EvNameId | Unset = UNSET
    fixed_cable: bool | Unset = UNSET
    max_power_level: float | Unset = UNSET
    charging_point: EvChargingPoint | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        supplier_name = self.supplier_name

        connector_type: dict[str, Any] | Unset = UNSET
        if not isinstance(self.connector_type, Unset):
            connector_type = self.connector_type.to_dict()

        fixed_cable = self.fixed_cable

        max_power_level = self.max_power_level

        charging_point: dict[str, Any] | Unset = UNSET
        if not isinstance(self.charging_point, Unset):
            charging_point = self.charging_point.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if supplier_name is not UNSET:
            field_dict["supplierName"] = supplier_name
        if connector_type is not UNSET:
            field_dict["connectorType"] = connector_type
        if fixed_cable is not UNSET:
            field_dict["fixedCable"] = fixed_cable
        if max_power_level is not UNSET:
            field_dict["maxPowerLevel"] = max_power_level
        if charging_point is not UNSET:
            field_dict["chargingPoint"] = charging_point

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.ev_charging_point import EvChargingPoint
        from ..models.ev_name_id import EvNameId

        d = dict(src_dict)
        supplier_name = d.pop("supplierName", UNSET)

        _connector_type = d.pop("connectorType", UNSET)
        connector_type: EvNameId | Unset
        if isinstance(_connector_type, Unset):
            connector_type = UNSET
        else:
            connector_type = EvNameId.from_dict(_connector_type)

        fixed_cable = d.pop("fixedCable", UNSET)

        max_power_level = d.pop("maxPowerLevel", UNSET)

        _charging_point = d.pop("chargingPoint", UNSET)
        charging_point: EvChargingPoint | Unset
        if isinstance(_charging_point, Unset):
            charging_point = UNSET
        else:
            charging_point = EvChargingPoint.from_dict(_charging_point)

        ev_connector = cls(
            supplier_name=supplier_name,
            connector_type=connector_type,
            fixed_cable=fixed_cable,
            max_power_level=max_power_level,
            charging_point=charging_point,
        )

        ev_connector.additional_properties = d
        return ev_connector

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.autocomplete_result_item_administrative_area_type import (
    AutocompleteResultItemAdministrativeAreaType,
)
from ..models.autocomplete_result_item_house_number_type import (
    AutocompleteResultItemHouseNumberType,
)
from ..models.autocomplete_result_item_locality_type import (
    AutocompleteResultItemLocalityType,
)
from ..models.autocomplete_result_item_result_type import (
    AutocompleteResultItemResultType,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.address import Address
    from ..models.street_info import StreetInfo
    from ..models.title_and_address_highlighting import TitleAndAddressHighlighting


T = TypeVar("T", bound="AutocompleteResultItem")


@_attrs_define
class AutocompleteResultItem:
    """
    Attributes:
        title (str): The unified display name of this result item. The result title is composed so that the customer
            application can use it to highlight parts of the suggestions. It is following a consistent schema
            for all results, regardless of their location. The highlighting details are about the
            title field and each of the address fields, from the country name down to the address house number.
            It is also built out from address components identified by the end-user, who can choose a result.
            For example: "Germany, 32547, Bad Oeynhausen, Schulstraße 4"
        id (str): The unique identifier for the result item. This ID can be used for a Look Up by ID search as well.
        address (Address):
        language (str | Unset): The preferred language of address elements in the result.
        political_view (str | Unset): ISO3 country code of the item political view (default for international). This
            response element is populated when the politicalView parameter is set in the query
        result_type (AutocompleteResultItemResultType | Unset):
            Type of the result item.

            Note: `addressBlock` result item is either a block or subblock.

            `resultType` values can get added to the list without further notice.
        house_number_type (AutocompleteResultItemHouseNumberType | Unset): Indicates the address type which affects the
            precision of the address and the coordinate accuracy.

            Description of supported values:

            - **RESTRICTED** `MPA`: A Micro Point Address represents a secondary address for a Point
              Address; for example, building, floor (level) and suite (unit).
              Micro Point Addresses can be used to enhance Point Address with
              greater address detail and higher coordinate accuracy. This result
              type is only returned by Lookup, Geocode or (Multi) Reverse Geocode endpoints when
              `with=MPA` parameter is provided.
            - `PA`: A Point Address represents an individual address as a point object.
              Point Addresses are coming from trusted sources. There is a
              high certainty that the address exists at that position. A
              Point Address result contains two types of coordinates. One is the
              access point (or navigation coordinates), which is the point to
              start or end a drive. The other point is the position or display
              point. This point varies per source and country. The point can be
              the rooftop point, a point close to the building entry, or a point
              close to the building, driveway or parking lot that belongs to the
              building.
            - `interpolated`: An interpolated address. These are approximate positions as a
              result of a linear interpolation based on address ranges. Address
              ranges, especially in the USA, are typical per block. For
              interpolated addresses, we cannot say with confidence that the
              address exists in reality. But the interpolation provides a good
              location approximation that brings people in most use cases close
              to the target location. The access point of an interpolated address
              result is calculated based on the address range and the road
              geometry. The position (display) point is pre-configured offset
              from the street geometry. Compared to Point Addresses, interpolated
              addresses are less accurate.
        estimated_point_address (bool | Unset): If true, indicates that the coordinates of `position` and `access`
            points of the Point Address are
            estimated.
            This field is visible only for result items with resultType `houseNumber` and houseNumberType `PA` and
            only when the value is `true`
        locality_type (AutocompleteResultItemLocalityType | Unset):
            Type of the locality result if the `resultType` field is set to `locality`.

            `localityType` values can get added to the list without further notice.
        administrative_area_type (AutocompleteResultItemAdministrativeAreaType | Unset):
            Type of the administrative area result if the `resultType` field is set to `administrativeArea`.

            `administrativeAreaType` values can get added to the list without further notice.
        distance (int | Unset): The distance "as the crow flies" from the search center to this result item in meters.
            For example: "172039".

            When searching along a route this is the distance along the route plus the distance from the route polyline to
            this result item. Example: 172039.
        highlights (TitleAndAddressHighlighting | Unset):
        has_related_mpa (bool | Unset): **ALPHA, RESTRICTED**

            `true`: Indicates that sub-premises (referred to as Micro Point Addresses in HERE terminology) are associated
            with the house number.

            `false`: Indicates that no sub-premises (referred to as Micro Point Addresses in HERE terminology) are
            associated with the house number.

            This field is visible only for result items with resultType `houseNumber` and houseNumberType `PA`.

            The field is rendered only if `show=hasRelatedMPA` is provided.
        street_info (list[StreetInfo] | Unset): Street Details (only rendered if `show=streetInfo` is provided.)
    """

    title: str
    id: str
    address: Address
    language: str | Unset = UNSET
    political_view: str | Unset = UNSET
    result_type: AutocompleteResultItemResultType | Unset = UNSET
    house_number_type: AutocompleteResultItemHouseNumberType | Unset = UNSET
    estimated_point_address: bool | Unset = UNSET
    locality_type: AutocompleteResultItemLocalityType | Unset = UNSET
    administrative_area_type: AutocompleteResultItemAdministrativeAreaType | Unset = (
        UNSET
    )
    distance: int | Unset = UNSET
    highlights: TitleAndAddressHighlighting | Unset = UNSET
    has_related_mpa: bool | Unset = UNSET
    street_info: list[StreetInfo] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        title = self.title

        id = self.id

        address = self.address.to_dict()

        language = self.language

        political_view = self.political_view

        result_type: str | Unset = UNSET
        if not isinstance(self.result_type, Unset):
            result_type = self.result_type.value

        house_number_type: str | Unset = UNSET
        if not isinstance(self.house_number_type, Unset):
            house_number_type = self.house_number_type.value

        estimated_point_address = self.estimated_point_address

        locality_type: str | Unset = UNSET
        if not isinstance(self.locality_type, Unset):
            locality_type = self.locality_type.value

        administrative_area_type: str | Unset = UNSET
        if not isinstance(self.administrative_area_type, Unset):
            administrative_area_type = self.administrative_area_type.value

        distance = self.distance

        highlights: dict[str, Any] | Unset = UNSET
        if not isinstance(self.highlights, Unset):
            highlights = self.highlights.to_dict()

        has_related_mpa = self.has_related_mpa

        street_info: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.street_info, Unset):
            street_info = []
            for street_info_item_data in self.street_info:
                street_info_item = street_info_item_data.to_dict()
                street_info.append(street_info_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "title": title,
            "id": id,
            "address": address,
        })
        if language is not UNSET:
            field_dict["language"] = language
        if political_view is not UNSET:
            field_dict["politicalView"] = political_view
        if result_type is not UNSET:
            field_dict["resultType"] = result_type
        if house_number_type is not UNSET:
            field_dict["houseNumberType"] = house_number_type
        if estimated_point_address is not UNSET:
            field_dict["estimatedPointAddress"] = estimated_point_address
        if locality_type is not UNSET:
            field_dict["localityType"] = locality_type
        if administrative_area_type is not UNSET:
            field_dict["administrativeAreaType"] = administrative_area_type
        if distance is not UNSET:
            field_dict["distance"] = distance
        if highlights is not UNSET:
            field_dict["highlights"] = highlights
        if has_related_mpa is not UNSET:
            field_dict["hasRelatedMPA"] = has_related_mpa
        if street_info is not UNSET:
            field_dict["streetInfo"] = street_info

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.address import Address
        from ..models.street_info import StreetInfo
        from ..models.title_and_address_highlighting import TitleAndAddressHighlighting

        d = dict(src_dict)
        title = d.pop("title")

        id = d.pop("id")

        address = Address.from_dict(d.pop("address"))

        language = d.pop("language", UNSET)

        political_view = d.pop("politicalView", UNSET)

        _result_type = d.pop("resultType", UNSET)
        result_type: AutocompleteResultItemResultType | Unset
        if isinstance(_result_type, Unset):
            result_type = UNSET
        else:
            result_type = AutocompleteResultItemResultType(_result_type)

        _house_number_type = d.pop("houseNumberType", UNSET)
        house_number_type: AutocompleteResultItemHouseNumberType | Unset
        if isinstance(_house_number_type, Unset):
            house_number_type = UNSET
        else:
            house_number_type = AutocompleteResultItemHouseNumberType(
                _house_number_type
            )

        estimated_point_address = d.pop("estimatedPointAddress", UNSET)

        _locality_type = d.pop("localityType", UNSET)
        locality_type: AutocompleteResultItemLocalityType | Unset
        if isinstance(_locality_type, Unset):
            locality_type = UNSET
        else:
            locality_type = AutocompleteResultItemLocalityType(_locality_type)

        _administrative_area_type = d.pop("administrativeAreaType", UNSET)
        administrative_area_type: AutocompleteResultItemAdministrativeAreaType | Unset
        if isinstance(_administrative_area_type, Unset):
            administrative_area_type = UNSET
        else:
            administrative_area_type = AutocompleteResultItemAdministrativeAreaType(
                _administrative_area_type
            )

        distance = d.pop("distance", UNSET)

        _highlights = d.pop("highlights", UNSET)
        highlights: TitleAndAddressHighlighting | Unset
        if isinstance(_highlights, Unset):
            highlights = UNSET
        else:
            highlights = TitleAndAddressHighlighting.from_dict(_highlights)

        has_related_mpa = d.pop("hasRelatedMPA", UNSET)

        _street_info = d.pop("streetInfo", UNSET)
        street_info: list[StreetInfo] | Unset = UNSET
        if _street_info is not UNSET:
            street_info = []
            for street_info_item_data in _street_info:
                street_info_item = StreetInfo.from_dict(street_info_item_data)

                street_info.append(street_info_item)

        autocomplete_result_item = cls(
            title=title,
            id=id,
            address=address,
            language=language,
            political_view=political_view,
            result_type=result_type,
            house_number_type=house_number_type,
            estimated_point_address=estimated_point_address,
            locality_type=locality_type,
            administrative_area_type=administrative_area_type,
            distance=distance,
            highlights=highlights,
            has_related_mpa=has_related_mpa,
            street_info=street_info,
        )

        autocomplete_result_item.additional_properties = d
        return autocomplete_result_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GetAutocompletePostalCodeMode(str, Enum):
    CITYLOOKUP = "cityLookup"
    DISTRICTLOOKUP = "districtLookup"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.extended_access_point_type import ExtendedAccessPointType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExtendedAccessPoint")


@_attrs_define
class ExtendedAccessPoint:
    """
    Attributes:
        lat (float): Latitude of the access position. For example: `"52.19404"`
        lng (float): Longitude of the access position. For example: `"8.80135"`
        primary (bool | Unset): Only applies to response items of type `"place"`. Set to true for the primary access
            position when the place
            result has more than one `access` position.
            Only the first element in the access array can have a primary field set to `true`.
        type_ (ExtendedAccessPointType | Unset): Only applies to response items of type `"place"`

            Description of supported values:

            - `delivery`: A designated access position for receiving goods and services, typically used by delivery
            personnel.
            - `emergency`: A position specifically designated for emergency access or services,
              such as ambulance or fire vehicle entry points.
            - `entrance`: A designated access position for larger or complex places, such as shopping malls or sports
            centers,
              where multiple entrances are located around the perimeter. Each entrance provides a specific
              point of entry for visitors.
            - `loading`: An access position to a designated area for the loading and unloading of goods, typically used by
            delivery and service vehicles.
            - `other`: An access type that does not fit into predefined categories, used for unique or less common
            situations.
            - `parking`: An access position to a designated private area for vehicle parking, associated with the place.
            - `taxi`: A position specifically designated for taxi/ride-share pick-up and drop-off services.
        label (str | Unset): Only applies to response items of type `"place"`. The value is a short textual description
            of the access point. Example: "North Entrance"
    """

    lat: float
    lng: float
    primary: bool | Unset = UNSET
    type_: ExtendedAccessPointType | Unset = UNSET
    label: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        lat = self.lat

        lng = self.lng

        primary = self.primary

        type_: str | Unset = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        label = self.label

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "lat": lat,
            "lng": lng,
        })
        if primary is not UNSET:
            field_dict["primary"] = primary
        if type_ is not UNSET:
            field_dict["type"] = type_
        if label is not UNSET:
            field_dict["label"] = label

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        lat = d.pop("lat")

        lng = d.pop("lng")

        primary = d.pop("primary", UNSET)

        _type_ = d.pop("type", UNSET)
        type_: ExtendedAccessPointType | Unset
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = ExtendedAccessPointType(_type_)

        label = d.pop("label", UNSET)

        extended_access_point = cls(
            lat=lat,
            lng=lng,
            primary=primary,
            type_=type_,
            label=label,
        )

        extended_access_point.additional_properties = d
        return extended_access_point

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ReferenceSupplierId(str, Enum):
    BOOKING_COM = "booking.com"
    CORE = "core"
    NSR = "nsr"
    PARKOPEDIA = "parkopedia"
    RYD = "ryd"
    TRIPADVISOR = "tripadvisor"
    VENUES = "venues"
    VINFAST = "vinfast"
    YELP = "yelp"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="AutosuggestBody")


@_attrs_define
class AutosuggestBody:
    r"""
    Attributes:
        route (str | Unset): **BETA**

            Select within a geographic corridor. This is a hard filter. Results will be returned if they are located within
            the specified area.

            A `route` is defined by a [Flexible Polyline Encoding](https://github.com/heremaps/flexible-polyline),
             followed by an optional width, represented by a sub-parameter "w".

            Format: `{route};w={width}`

            In regular expression syntax, the values of `route` are formatted as follows:

            `[a-zA-Z0-9_-]+(;w=\d+)?`

            "[a-zA-Z0-9._-]+" is the encoded flexible polyline.

            "w=\d+" is the optional width. The width is specified in meters from the center of the path. If no width is
            provided, the default is 1000 meters.

            Type: `{Flexible Polyline Encoding};w={integer}`

            The following constraints apply:
             * A `route` MUST contain at least two points (one segment).
             * A `route` MUST NOT contain more than 2,000 points.
             * A `route` MUST NOT have a width of more than 50,000 meters.

            Examples:
             * `BFoz5xJ67i1B1B7PzIhaxL7Y`
             * `BFoz5xJ67i1B1B7PzIhaxL7Y;w=5000`
             * `BlD05xgKuy2xCCx9B7vUCl0OhnRC54EqSCzpEl-HCxjD3pBCiGnyGCi2CvwFCsgD3nDC4vB6eC;w=2000`

            Note: The last example above can be decoded (using the Python class [here](https://github.com/heremaps/flexible-
            polyline/tree/master/python) as follows:

            ```
            >>> import flexpolyline
            >>> polyline = 'BlD05xgKuy2xCCx9B7vUCl0OhnRC54EqSCzpEl-HCxjD3pBCiGnyGCi2CvwFCsgD3nDC4vB6eC'
            >>> flexpolyline.decode(polyline)
            [(52.51994, 13.38663, 1.0), (52.51009, 13.28169, 2.0), (52.43518, 13.19352, 3.0), (52.41073, 13.19645, 4.0),
             (52.38871, 13.15578, 5.0), (52.37278, 13.1491, 6.0), (52.37375, 13.11546, 7.0), (52.38752, 13.08722, 8.0),
             (52.40294, 13.07062, 9.0), (52.41058, 13.07555, 10.0)]
            ```
    """

    route: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        route = self.route

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if route is not UNSET:
            field_dict["route"] = route

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        route = d.pop("route", UNSET)

        autosuggest_body = cls(
            route=route,
        )

        autosuggest_body.additional_properties = d
        return autosuggest_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.signpost_label_route_number import SignpostLabelRouteNumber
    from ..models.signpost_label_text import SignpostLabelText


T = TypeVar("T", bound="SignpostInfo")


@_attrs_define
class SignpostInfo:
    """Signpost information attached to an offset action.

    Example:
        {'$ref': '#/components/examples/routeResponseManeuverSignpostInfoExample'}

    Attributes:
        labels (list[SignpostLabelRouteNumber | SignpostLabelText]): Part of a signpost representing particular
            direction or destination.
    """

    labels: list[SignpostLabelRouteNumber | SignpostLabelText]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.signpost_label_text import SignpostLabelText

        labels = []
        for labels_item_data in self.labels:
            labels_item: dict[str, Any]
            if isinstance(labels_item_data, SignpostLabelText):
                labels_item = labels_item_data.to_dict()
            else:
                labels_item = labels_item_data.to_dict()

            labels.append(labels_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "labels": labels,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.signpost_label_route_number import SignpostLabelRouteNumber
        from ..models.signpost_label_text import SignpostLabelText

        d = dict(src_dict)
        labels = []
        _labels = d.pop("labels")
        for labels_item_data in _labels:

            def _parse_labels_item(
                data: object,
            ) -> SignpostLabelRouteNumber | SignpostLabelText:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_signpost_label_type_0 = (
                        SignpostLabelText.from_dict(data)
                    )

                    return componentsschemas_signpost_label_type_0
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_signpost_label_type_1 = (
                    SignpostLabelRouteNumber.from_dict(data)
                )

                return componentsschemas_signpost_label_type_1

            labels_item = _parse_labels_item(labels_item_data)

            labels.append(labels_item)

        signpost_info = cls(
            labels=labels,
        )

        signpost_info.additional_properties = d
        return signpost_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
